var proxmoxOnlineHelpInfo = {
    "pbs_documentation_index" : {
	"link" : "/pbs-docs/index.html",
	"title" : "Proxmox Backup Server Documentation Index"
    }
};
Ext.define('PBS.form.UserSelector', {
    extend: 'Proxmox.form.ComboGrid',
    alias: 'widget.pbsUserSelector',

    allowBlank: false,
    autoSelect: false,
    valueField: 'userid',
    displayField: 'userid',

    editable: true,
    anyMatch: true,
    forceSelection: true,

    store: {
	model: 'pmx-users',
	autoLoad: true,
	params: {
	    enabled: 1,
	},
	sorters: 'userid',
    },

    listConfig: {
	columns: [
	    {
		header: gettext('User'),
		sortable: true,
		dataIndex: 'userid',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	    {
		header: gettext('Name'),
		sortable: true,
		renderer: (first, mD, rec) => Ext.String.htmlEncode(
		    `${first || ''} ${rec.data.lastname || ''}`,
		),
		dataIndex: 'firstname',
		flex: 1,
	    },
	    {
		header: gettext('Comment'),
		sortable: false,
		dataIndex: 'comment',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	],
    },
});
Ext.define('PBS.form.RemoteSelector', {
    extend: 'Proxmox.form.ComboGrid',
    alias: 'widget.pbsRemoteSelector',

    allowBlank: false,
    autoSelect: false,
    valueField: 'name',
    displayField: 'name',

    store: {
	model: 'pmx-remotes',
	autoLoad: true,
	sorters: 'name',
    },

    listConfig: {
	columns: [
	    {
		header: gettext('Remote'),
		sortable: true,
		dataIndex: 'name',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	    {
		header: gettext('Host'),
		sortable: true,
		dataIndex: 'host',
		flex: 1,
	    },
	    {
		header: gettext('User name'),
		sortable: true,
		dataIndex: 'userid',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	],
    },
});
Ext.define('PBS.form.DataStoreSelector', {
    extend: 'Proxmox.form.ComboGrid',
    alias: 'widget.pbsDataStoreSelector',

    allowBlank: false,
    autoSelect: false,
    valueField: 'store',
    displayField: 'store',

    store: {
	model: 'pbs-datastore-list',
	autoLoad: true,
	sorters: 'store',
    },

    listConfig: {
	columns: [
	    {
		header: gettext('Datastore'),
		sortable: true,
		dataIndex: 'store',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	    {
		header: gettext('Comment'),
		sortable: true,
		dataIndex: 'comment',
		renderer: Ext.String.htmlEncode,
		flex: 1,
	    },
	],
    },
});
Ext.define('PBS.data.CalendarEventExamples', {
    extend: 'Ext.data.Store',
    alias: 'store.calendarEventExamples',

    field: ['value', 'text'],
    data: [
	//FIXME { value: '*/30', text: Ext.String.format(gettext("Every {0} minutes"), 30) },
	{ value: 'hourly', text: gettext("Every hour") },
	//FIXME { value: '*/2:00', text: gettext("Every two hours") },
	{ value: '2,22:30', text: gettext("Every day") + " 02:30, 22:30" },
	{ value: 'daily', text: gettext("Every day") + " 00:00" },
	{ value: 'mon..fri', text: gettext("Monday to Friday") + " 00:00" },
	//FIXME{ value: 'mon..fri */1:00', text: gettext("Monday to Friday") + ': ' + gettext("hourly") },
	{ value: 'sat 18:15', text: gettext("Every Saturday") + " 18:15" },
	//FIXME{ value: 'monthly', text: gettext("Every 1st of Month") + " 00:00" }, // not yet possible..
    ],
});

Ext.define('PBS.form.CalendarEvent', {
    extend: 'Ext.form.field.ComboBox',
    xtype: 'pbsCalendarEvent',

    editable: true,

    valueField: 'value',
    displayField: 'text',
    queryMode: 'local',

    config: {
	deleteEmpty: true,
    },
    // overide framework function to implement deleteEmpty behaviour
    getSubmitData: function() {
	let me = this, data = null;
	if (!me.disabled && me.submitValue) {
	    let val = me.getSubmitValue();
	    if (val !== null && val !== '' && val !== '__default__') {
		data = {};
		data[me.getName()] = val;
	    } else if (me.getDeleteEmpty()) {
		data = {};
		data.delete = me.getName();
	    }
	}
	return data;
    },


    store: {
	type: 'calendarEventExamples',
    },

    tpl: [
	'<ul class="x-list-plain"><tpl for=".">',
	    '<li role="option" class="x-boundlist-item">{text}</li>',
	'</tpl></ul>',
    ],

    displayTpl: [
	'<tpl for=".">',
	'{value}',
	'</tpl>',
    ],
});
Ext.define('PBS.data.RunningTasksStore', {
    extend: 'Proxmox.data.UpdateStore',

    singleton: true,

    constructor: function(config) {
	let me = this;
	config = config || {};
	Ext.apply(config, {
	    interval: 3000,
	    storeid: 'pbs-running-tasks-dash',
	    model: 'proxmox-tasks',
	    proxy: {
		type: 'proxmox',
		// maybe separate api call?
		url: '/api2/json/nodes/localhost/tasks?running=1&limit=100',
	    },
	});
	me.callParent([config]);
    },
});
Ext.define('PBS.TaskButton', {
    extend: 'Ext.button.Button',
    alias: 'widget.pbsTaskButton',

    config: {
	badgeText: '0',
	badgeCls: '',
    },

    iconCls: 'fa fa-list',
    userCls: 'pmx-has-badge',
    text: gettext('Tasks'),

    setText: function(value) {
	let me = this;
	me.realText = value;
	let badgeText = me.getBadgeText();
	let badgeCls = me.getBadgeCls();
	let text = `${value} <span class="pmx-button-badge ${badgeCls}">${badgeText}</span>`;
	return me.callParent([text]);
    },

    getText: function() {
	let me = this;
	return me.realText;
    },

    setBadgeText: function(value) {
	let me = this;
	me.badgeText = value.toString();
	return me.setText(me.getText());
    },

    setBadgeCls: function(value) {
	let me = this;
	let res = me.callParent([value]);
	let badgeText = me.getBadgeText();
	me.setBadgeText(badgeText);
	return res;
    },

    handler: function() {
	let me = this;
	if (me.grid.isVisible()) {
	    me.grid.setVisible(false);
	} else {
	    me.grid.showBy(me, 'tr-br');
	}
    },

    initComponent: function() {
	let me = this;

	me.grid = Ext.create({
	    xtype: 'pbsRunningTasks',
	    title: '',
	    hideHeaders: false,
	    floating: true,

	    width: 600,

	    bbar: [
		'->',
		{
		    xtype: 'button',
		    text: gettext('Show All Tasks'),
		    handler: function() {
			var mainview = me.up('mainview');
			mainview.getController().redirectTo('pbsServerAdministration:tasks');
			me.grid.hide();
		    },
		},
	    ],

	    listeners: {
		'taskopened': function() {
		    me.grid.hide();
		},
	    },
	});
	me.callParent();
	me.mon(me.grid.getStore().rstore, 'load', function(store, records, success) {
	    if (!success) return;

	    let count = records.length;
	    let text = count > 99 ? '99+' : count.toString();
	    let cls = count > 0 ? 'active': '';
	    me.setBadgeText(text);
	    me.setBadgeCls(cls);
	});
    },
});
Ext.define('pmx-users', {
    extend: 'Ext.data.Model',
    fields: [
	'userid', 'firstname', 'lastname', 'email', 'comment',
	{ type: 'boolean', name: 'enable', defaultValue: true },
	{ type: 'date', dateFormat: 'timestamp', name: 'expire' },
    ],
    idProperty: 'userid',
    proxy: {
	type: 'proxmox',
	url: '/api2/json/access/users',
    },
});

Ext.define('PBS.config.UserView', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pbsUserView',

    stateful: true,
    stateId: 'grid-users',

    title: gettext('Users'),

    controller: {
	xclass: 'Ext.app.ViewController',

	addUser: function() {
	    let me = this;
            Ext.create('PBS.window.UserEdit', {
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	editUser: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

            Ext.create('PBS.window.UserEdit', {
                userid: selection[0].data.userid,
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	renderUsername: function(userid) {
	    return Ext.String.htmlEncode(userid.match(/^(.+)@([^@]+)$/)[1]);
	},

	renderRealm: function(userid) {
	    return Ext.String.htmlEncode(userid.match(/^(.+)@([^@]+)$/)[2]);
	},

	reload: function() { this.getView().getStore().rstore.load(); },

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
    },

    listeners: {
	activate: 'reload',
	itemdblclick: 'editUser',
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'userid',
	rstore: {
	    type: 'update',
	    storeid: 'pmx-users',
	    model: 'pmx-users',
	    autoStart: true,
	    interval: 5000,
	},
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Add'),
	    handler: 'addUser',
	    selModel: false,
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    handler: 'editUser',
	    disabled: true,
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/access/users/',
	    enableFn: (rec) => rec.data.userid !== 'root@pam',
	    callback: 'reload',
	},
    ],

    viewConfig: {
	trackOver: false,
    },

    columns: [
	{
	    header: gettext('User name'),
	    width: 200,
	    sortable: true,
	    renderer: 'renderUsername',
	    dataIndex: 'userid',
	},
	{
	    header: gettext('Realm'),
	    width: 100,
	    sortable: true,
	    renderer: 'renderRealm',
	    dataIndex: 'userid',
	},
	{
	    header: gettext('Enabled'),
	    width: 80,
	    sortable: true,
	    renderer: Proxmox.Utils.format_boolean,
	    dataIndex: 'enable',
	},
	{
	    header: gettext('Expire'),
	    width: 80,
	    sortable: true,
	    renderer: Proxmox.Utils.format_expire,
	    dataIndex: 'expire',
	},
	{
	    header: gettext('Name'),
	    width: 150,
	    sortable: true,
	    dataIndex: 'firstname',
	},
	{
	    header: gettext('Comment'),
	    sortable: false,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'comment',
	    flex: 1,
	},
    ],
});
Ext.define('pmx-remotes', {
    extend: 'Ext.data.Model',
    fields: [ 'name', 'host', 'userid', 'fingerprint', 'comment' ],
    idProperty: 'name',
    proxy: {
	type: 'proxmox',
	url: '/api2/json/config/remote',
    },
});

Ext.define('PBS.config.RemoteView', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pbsRemoteView',

    stateful: true,
    stateId: 'grid-remotes',

    title: gettext('Remotes'),

    controller: {
	xclass: 'Ext.app.ViewController',

	addRemote: function() {
	    let me = this;
            Ext.create('PBS.window.RemoteEdit', {
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	editRemote: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

            Ext.create('PBS.window.RemoteEdit', {
                name: selection[0].data.name,
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	reload: function() { this.getView().getStore().rstore.load(); },

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
    },

    listeners: {
	activate: 'reload',
	itemdblclick: 'editRemote',
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'name',
	rstore: {
	    type: 'update',
	    storeid: 'pmx-remotes',
	    model: 'pmx-remotes',
	    autoStart: true,
	    interval: 5000,
	},
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Add'),
	    handler: 'addRemote',
	    selModel: false,
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    handler: 'editRemote',
	    disabled: true,
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/config/remote',
	    callback: 'reload',
	},
    ],

    viewConfig: {
	trackOver: false,
    },

    columns: [
	{
	    header: gettext('Remote'),
	    width: 200,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'name',
	},
	{
	    header: gettext('Host'),
	    width: 200,
	    sortable: true,
	    dataIndex: 'host',
	},
	{
	    header: gettext('User name'),
	    width: 200,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'userid',
	},
	{
	    header: gettext('Fingerprint'),
	    sortable: false,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'fingerprint',
	    width: 200,
	},
	{
	    header: gettext('Comment'),
	    sortable: false,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'comment',
	    flex: 1,
	},
    ],
});
Ext.define('pmx-acls', {
    extend: 'Ext.data.Model',
    fields: [
	'path', 'ugid', 'ugid_type', 'roleid', 'propagate',
	{
	    name: 'aclid',
	    calculate: function(data) {
		return `${data.path} for ${data.ugid} - ${data.roleid}`;
	    },
	},
    ],
    idProperty: 'aclid',
    proxy: {
	type: 'proxmox',
	url: '/api2/json/access/acl',
    },
});

Ext.define('PBS.config.ACLView', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pbsACLView',

    stateful: true,
    stateId: 'grid-acls',

    title: gettext('Permissions'),

    aclPath: undefined,
    aclExact: undefined,

    controller: {
	xclass: 'Ext.app.ViewController',

	addACL: function() {
	    let me = this;
	    let view = me.getView();
            Ext.create('PBS.window.ACLEdit', {
		path: view.aclPath,
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	removeACL: function(btn, event, rec) {
	    let me = this;
	    Proxmox.Utils.API2Request({
		url:'/access/acl',
		method: 'PUT',
		params: {
		    'delete': 1,
		    path: rec.data.path,
		    role: rec.data.roleid,
		    userid: rec.data.ugid,
		},
		callback: function() {
		    me.reload();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
	    });
	},

	reload: function() { this.getView().getStore().rstore.load(); },

	init: function(view) {
	    let proxy = view.getStore().rstore.getProxy();

	    let params = {};
	    if (view.aclPath !== undefined) {
		params.path = view.aclPath;
	    }
	    if (view.aclExact !== undefined) {
		params.exact = view.aclExact;
	    }
	    proxy.setExtraParams(params);
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
	control: {
	    '#': { // view
		activate: function() {
		    this.getView().getStore().rstore.startUpdate();
		},
		deactivate: function() {
		    this.getView().getStore().rstore.stopUpdate();
		},
	    },
	},
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'aclid',
	rstore: {
	    type: 'update',
	    storeid: 'pmx-acls',
	    model: 'pmx-acls',
	    interval: 5000,
	},
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Add'),
	    handler: 'addACL',
	    selModel: false,
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    handler: 'removeACL',
	    callback: 'reload',
	},
    ],

    columns: [
	{
	    header: gettext('Path'),
	    width: 200,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'path',
	},
	{
	    header: gettext('User/Group'),
	    width: 100,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'ugid',
	},
	{
	    header: gettext('Role'),
	    width: 80,
	    sortable: true,
	    dataIndex: 'roleid',
	},
	{
	    header: gettext('Propagate'),
	    width: 150,
	    sortable: true,
	    renderer: Proxmox.Utils.format_boolean,
	    dataIndex: 'propagate',
	},
    ],

});
Ext.define('pbs-sync-jobs-status', {
    extend: 'Ext.data.Model',
    fields: [
	'id', 'remote', 'remote-store', 'store', 'schedule',
	'next-run', 'last-run-upid', 'last-run-state', 'last-run-endtime',
	{
	    name: 'duration',
	    calculate: function(data) {
		let endtime = data['last-run-endtime'];
		if (!endtime) return undefined;
		let task = Proxmox.Utils.parse_task_upid(data['last-run-upid']);
		return endtime - task.starttime;
	    },
	},
    ],
    idProperty: 'id',
    proxy: {
	type: 'proxmox',
	url: '/api2/json/admin/sync',
    },
});

Ext.define('PBS.config.SyncJobView', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pbsSyncJobView',

    stateful: true,
    stateId: 'grid-sync-jobs',

    title: gettext('Sync Jobs'),

    controller: {
	xclass: 'Ext.app.ViewController',

	addSyncJob: function() {
	    let me = this;
            Ext.create('PBS.window.SyncJobEdit', {
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	editSyncJob: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

            Ext.create('PBS.window.SyncJobEdit', {
                id: selection[0].data.id,
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
            }).show();
	},

	openTaskLog: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

	    let upid = selection[0].data['last-run-upid'];
	    if (!upid) return;

	    Ext.create('Proxmox.window.TaskViewer', {
		upid
	    }).show();
	},

	runSyncJob: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

	    let id = selection[0].data.id;
	    Proxmox.Utils.API2Request({
		method: 'POST',
		url: `/admin/sync/${id}/run`,
		success: function(response, opt) {
		    Ext.create('Proxmox.window.TaskViewer', {
		        upid: response.result.data,
		        taskDone: function(success) {
			    me.reload();
		        },
		    }).show();
		},
		failure: function(response, opt) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
	    });
	},

	render_sync_status: function(value, metadata, record) {
	    if (!record.data['last-run-upid']) {
		return '-';
	    }

	    if (!record.data['last-run-endtime']) {
		metadata.tdCls = 'x-grid-row-loading';
		return '';
	    }

	    if (value === 'OK') {
		return `<i class="fa fa-check good"></i> ${gettext("OK")}`;
	    }

	    return `<i class="fa fa-times critical"></i> ${gettext("Error")}:${value}`;
	},

	render_next_run: function(value, metadat, record) {
	    if (!value) return '-';

	    let now = new Date();
	    let next = new Date(value*1000);

	    if (next < now) {
		return gettext('pending');
	    }
	    return Proxmox.Utils.render_timestamp(value);
	},

	render_optional_timestamp: function(value, metadata, record) {
	    if (!value) return '-';
	    return Proxmox.Utils.render_timestamp(value);
	},

	reload: function() { this.getView().getStore().rstore.load(); },

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
    },

    listeners: {
	activate: 'reload',
	itemdblclick: 'editSyncJob',
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'id',
	rstore: {
	    type: 'update',
	    storeid: 'pbs-sync-jobs-status',
	    model: 'pbs-sync-jobs-status',
	    autoStart: true,
	    interval: 5000,
	},
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Add'),
	    handler: 'addSyncJob',
	    selModel: false,
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    handler: 'editSyncJob',
	    disabled: true,
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/config/sync/',
	    callback: 'reload',
	},
	'-',
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Log'),
	    handler: 'openTaskLog',
	    enableFn: (rec) => !!rec.data['last-run-upid'],
	    disabled: true,
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Run now'),
	    handler: 'runSyncJob',
	    disabled: true,
	},
    ],

    viewConfig: {
	trackOver: false,
    },

    columns: [
	{
	    header: gettext('Sync Job'),
	    width: 200,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'id',
	},
	{
	    header: gettext('Remote'),
	    width: 200,
	    sortable: true,
	    dataIndex: 'remote',
	},
	{
	    header: gettext('Remote Store'),
	    width: 200,
	    sortable: true,
	    dataIndex: 'remote-store',
	},
	{
	    header: gettext('Local Store'),
	    width: 200,
	    sortable: true,
	    dataIndex: 'store',
	},
	{
	    header: gettext('Schedule'),
	    sortable: true,
	    dataIndex: 'schedule',
	},
	{
	    header: gettext('Status'),
	    dataIndex: 'last-run-state',
	    flex: 1,
	    renderer: 'render_sync_status',
	},
	{
	    header: gettext('Last Sync'),
	    sortable: true,
	    minWidth: 200,
	    renderer: 'render_optional_timestamp',
	    dataIndex: 'last-run-endtime',
	},
	{
	    text: gettext('Duration'),
	    dataIndex: 'duration',
	    width: 60,
	    renderer: Proxmox.Utils.render_duration,
	},
	{
	    header: gettext('Next Run'),
	    sortable: true,
	    minWidth: 200,
	    renderer: 'render_next_run',
	    dataIndex: 'next-run',
	},
	{
	    header: gettext('Comment'),
	    hidden: true,
	    sortable: true,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'comment',
	},
    ],
});
Ext.define('pbs-datastore-list', {
    extend: 'Ext.data.Model',
    fields: [ 'name', 'comment' ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/admin/datastore"
    },
    idProperty: 'store'
});

Ext.define('pbs-data-store-config', {
    extend: 'Ext.data.Model',
    fields: [
	'name', 'path', 'comment', 'gc-schedule', 'prune-schedule', 'keep-last',
	'keep-hourly', 'keep-daily', 'keep-weekly', 'keep-monthly', 'keep-yearly',
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/datastore",
    },
    idProperty: 'name',
});

Ext.define('PBS.DataStoreConfig', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pbsDataStoreConfig',

    title: gettext('Datastore Configuration'),

    controller: {
	xclass: 'Ext.app.ViewController',

	createDataStore: function() {
	    let me = this;
	    Ext.create('PBS.DataStoreEdit', {
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
	    }).show();
	},

	editDataStore: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

	    let name = encodeURIComponent(selection[0].data.name);
	    Ext.create('PBS.DataStoreEdit', {
		name: name,
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
	    }).show();
	},

	onVerify: function() {
	    var view = this.getView();

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;

	    Proxmox.Utils.API2Request({
		url: `/admin/datastore/${data.name}/verify`,
		method: 'POST',
		failure: function(response) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, options) {
		    Ext.create('Proxmox.window.TaskViewer', {
			upid: response.result.data,
		    }).show();
		},
	    });
	},

	garbageCollect: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (selection.length < 1) return;

	    let name = encodeURIComponent(selection[0].data.name);
	    Proxmox.Utils.API2Request({
		url: `/admin/datastore/${name}/gc`,
		method: 'POST',
		failure: function(response) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, options) {
		    Ext.create('Proxmox.window.TaskViewer', {
			upid: response.result.data,
		    }).show();
		},
	    });
	},

	reload: function() { this.getView().getStore().rstore.load(); },

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'name',
	rstore: {
	    type: 'update',
	    storeid: 'pbs-data-store-config',
	    model: 'pbs-data-store-config',
	    autoStart: true,
	    interval: 10000,
	},
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    selModel: false,
	    text: gettext('Create'),
	    handler: 'createDataStore',
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    disabled: true,
	    handler: 'editDataStore',
	},
	// remove_btn
	'-',
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Verify'),
	    disabled: true,
	    handler: 'onVerify',
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Start GC'),
	    disabled: true,
	    handler: 'garbageCollect',
	},
    ],

    columns: [
	{
	    header: gettext('Name'),
	    sortable: true,
	    dataIndex: 'name',
	    flex: 1,
	},
	{
	    header: gettext('Path'),
	    sortable: true,
	    dataIndex: 'path',
	    flex: 1,
	},
	{
	    header: gettext('GC Schedule'),
	    sortable: false,
	    width: 120,
	    dataIndex: 'gc-schedule',
	},
	{
	    header: gettext('Prune Schedule'),
	    sortable: false,
	    width: 120,
	    dataIndex: 'prune-schedule',
	},
	{
	    header: gettext('Keep'),
	    columns: [
		{
		    text: gettext('Last'),
		    dataIndex: 'keep-last',
		    width: 70,
		},
		{
		    text: gettext('Hourly'),
		    dataIndex: 'keep-hourly',
		    width: 70,
		},
		{
		    text: gettext('Daily'),
		    dataIndex: 'keep-daily',
		    width: 70,
		},
		{
		    text: gettext('Weekly'),
		    dataIndex: 'keep-weekly',
		    width: 70,
		},
		{
		    text: gettext('Monthly'),
		    dataIndex: 'keep-monthly',
		    width: 70,
		},
		{
		    text: gettext('Yearly'),
		    dataIndex: 'keep-yearly',
		    width: 70,
		},
	    ]
	},
	{
	    header: gettext('Comment'),
	    sortable: false,
	    dataIndex: 'comment',
	    renderer: Ext.String.htmlEncode,
	    flex: 2,
	},
    ],

    listeners: {
	activate: 'reload',
	itemdblclick: 'editDataStore',
    },
});
Ext.define('PBS.window.UserEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pbsUserEdit',
    mixins: ['Proxmox.Mixin.CBind'],

    userid: undefined,

    isAdd: true,

    subject: gettext('User'),

    fieldDefaults: { labelWidth: 120 },

    cbindData: function(initialConfig) {
	var me = this;

	var userid = initialConfig.userid;
	var baseurl = '/api2/extjs/access/users';

	me.isCreate = !userid;
	me.url = userid ? baseurl + '/' + userid : baseurl;
	me.method = userid ? 'PUT' : 'POST';
	me.autoLoad = !!userid;

	return {
	    useridXType: userid ? 'displayfield' : 'textfield',
	};
    },

    items: {
	xtype: 'inputpanel',
	column1: [
	    {
		xtype: 'pmxDisplayEditField',
		name: 'userid',
		fieldLabel: gettext('User name'),
		renderer: Ext.htmlEncode,
		allowBlank: false,
		minLength: 4,
		cbind: {
		    editable: '{isCreate}',
		},
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		fieldLabel: gettext('Password'),
		minLength: 5,
		allowBlank: false,
		name: 'password',
		listeners: {
                    change: function(field) {
			field.next().validate();
                    },
                    blur: function(field) {
			field.next().validate();
                    },
		},
		cbind: {
		    hidden: '{!isCreate}',
		    disabled: '{!isCreate}',
		},
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		fieldLabel: gettext('Confirm password'),
		name: 'verifypassword',
		vtype: 'password',
		initialPassField: 'password',
		allowBlank: false,
		submitValue: false,
		cbind: {
		    hidden: '{!isCreate}',
		    disabled: '{!isCreate}',
		},
	    },
	    {
                xtype: 'datefield',
                name: 'expire',
		emptyText: Proxmox.Utils.neverText,
		format: 'Y-m-d',
		submitFormat: 'U',
                fieldLabel: gettext('Expire'),
            },
	    {
		xtype: 'proxmoxcheckbox',
		fieldLabel: gettext('Enabled'),
		name: 'enable',
		uncheckedValue: 0,
		defaultValue: 1,
		checked: true,
	    },
	],

	column2: [
	    {
		xtype: 'proxmoxtextfield',
		name: 'firstname',
		fieldLabel: gettext('First Name'),
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'lastname',
		fieldLabel: gettext('Last Name'),
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'email',
		fieldLabel: gettext('E-Mail'),
		vtype: 'proxmoxMail',
	    },
	],

	columnB: [
	    {
		xtype: 'proxmoxtextfield',
		name: 'comment',
		fieldLabel: gettext('Comment'),
	    },
	],
    },

    getValues: function(dirtyOnly) {
	var me = this;

	var values = me.callParent(arguments);

	// hack: ExtJS datefield does not submit 0, so we need to set that
	if (!values.expire) {
	    values.expire = 0;
	}

	if (me.isCreate) {
	    values.userid = values.userid + '@pbs';
	}

	delete values.username;

	if (!values.password) {
	    delete values.password;
	}

	return values;
    },

    setValues: function(values) {
	var me = this;

	if (Ext.isDefined(values.expire)) {
	    if (values.expire) {
		values.expire = new Date(values.expire * 1000);
	    } else {
		// display 'never' instead of '1970-01-01'
		values.expire = null;
	    }
	}

	me.callParent([values]);
    },
});
Ext.define('PBS.window.RemoteEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pbsRemoteEdit',
    mixins: ['Proxmox.Mixin.CBind'],

    userid: undefined,

    isAdd: true,

    subject: gettext('Remote'),

    fieldDefaults: { labelWidth: 120 },

    cbindData: function(initialConfig) {
	let me = this;

	let baseurl = '/api2/extjs/config/remote';
	let name = initialConfig.name;

	me.isCreate = !name;
	me.url = name ? `${baseurl}/${name}` : baseurl;
	me.method = name ? 'PUT' : 'POST';
	me.autoLoad = !!name;
	return {
	    passwordEmptyText: me.isCreate ? '' : gettext('Unchanged'),
	};
    },

    items: {
	xtype: 'inputpanel',
	column1: [
	    {
		xtype: 'pmxDisplayEditField',
		name: 'name',
		fieldLabel: gettext('Remote'),
		renderer: Ext.htmlEncode,
		allowBlank: false,
		minLength: 4,
		cbind: {
		    editable: '{isCreate}',
		},
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: false,
		name: 'host',
		fieldLabel: gettext('Host'),
	    },
	],

	column2: [
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: false,
		name: 'userid',
		fieldLabel: gettext('Userid'),
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		fieldLabel: gettext('Password'),
		name: 'password',
		cbind: {
		    emptyText: '{passwordEmptyText}',
		    allowBlank: '{!isCreate}',
		},
	    },
	],

	columnB: [
	    {
		xtype: 'proxmoxtextfield',
		name: 'fingerprint',
		fieldLabel: gettext('Fingerprint'),
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'comment',
		fieldLabel: gettext('Comment'),
	    },
	],
    },

    getValues: function() {
	let me = this;
	let values = me.callParent(arguments);

	if (values.password === '') {
	    delete values.password;
	}

	return values;
    },
});
Ext.define('PBS.window.SyncJobEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pbsSyncJobEdit',
    mixins: ['Proxmox.Mixin.CBind'],

    userid: undefined,

    isAdd: true,

    subject: gettext('SyncJob'),

    fieldDefaults: { labelWidth: 120 },

    cbindData: function(initialConfig) {
	let me = this;

	let baseurl = '/api2/extjs/config/sync';
	let id = initialConfig.id;

	me.isCreate = !id;
	me.url = id ? `${baseurl}/${id}` : baseurl;
	me.method = id ? 'PUT' : 'POST';
	me.autoLoad = !!id;
	return { };
    },

    items: {
	xtype: 'inputpanel',
	column1: [
	    {
		fieldLabel: gettext('Sync Job ID'),
		xtype: 'pmxDisplayEditField',
		name: 'id',
		renderer: Ext.htmlEncode,
		allowBlank: false,
		minLength: 4,
		cbind: {
		    editable: '{isCreate}',
		},
	    },
	    {
		fieldLabel: gettext('Source Remote'),
		xtype: 'pbsRemoteSelector',
		allowBlank: false,
		name: 'remote',
	    },
	    {
		fieldLabel: gettext('Source Datastore'),
		xtype: 'proxmoxtextfield',
		allowBlank: false,
		name: 'remote-store',
	    },
	    {
		fieldLabel: gettext('Local Datastore'),
		xtype: 'pbsDataStoreSelector',
		allowBlank: false,
		name: 'store',
	    },
	],

	column2: [
	    {
		fieldLabel: gettext('Remove vanished'),
		xtype: 'proxmoxcheckbox',
		name: 'remove-vanished',
		uncheckedValue: false,
		value: false,
	    },
	    {
		fieldLabel: gettext('Schedule'),
		xtype: 'pbsCalendarEvent',
		name: 'schedule',
		emptyText: gettext('none'),
		cbind: {
		    deleteEmpty: '{!isCreate}',
		},
	    },
	],

	columnB: [
	    {
		fieldLabel: gettext('Comment'),
		xtype: 'proxmoxtextfield',
		name: 'comment',
		cbind: {
		    deleteEmpty: '{!isCreate}',
		},
	    },
	],
    },
});
Ext.define('PBS.window.ACLEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pbsACLAdd',
    mixins: ['Proxmox.Mixin.CBind'],

    url: '/access/acl',
    method: 'PUT',
    isAdd: true,
    isCreate: true,

    // caller can give a static path
    path: undefined,

    subject: gettext('User Permission'),

    getValues: function(dirtyOnly) {
	let me = this;
	let values = me.callParent(arguments);

	if (me.path) {
	    values.path = me.path;
	}
	return values;
    },

    items: [
	{
	    xtype: 'pmxDisplayEditField',
	    fieldLabel: gettext('Path'),
	    cbind: {
		editable: '{!path}',
		value: '{path}',
	    },
	    name: 'path',
	    allowBlank: false,
	},
	{
	    xtype: 'pbsUserSelector',
	    fieldLabel: gettext('User'),
	    name: 'userid',
	    allowBlank: false,
	},
	{
	    xtype: 'pmxRoleSelector',
	    name: 'role',
	    value: 'NoAccess',
	    fieldLabel: gettext('Role'),
	},
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'propagate',
	    checked: true,
	    uncheckedValue: 0,
	    fieldLabel: gettext('Propagate'),
	},
    ],
});
Ext.define('PBS.DataStoreEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pbsDataStoreEdit',
    mixins: ['Proxmox.Mixin.CBind'],

    subject: gettext('Datastore'),
    isAdd: true,

    bodyPadding: 0,

    cbindData: function(initialConfig) {
	var me = this;

	let name = initialConfig.name;
	let baseurl = '/api2/extjs/config/datastore';

	me.isCreate = !name;
	if (!me.isCreate) {
	    me.defaultFocus = 'textfield[name=comment]';
	}
	me.url = name ? baseurl + '/' + name : baseurl;
	me.method = name ? 'PUT' : 'POST';
	me.autoLoad = !!name;
	return {};
    },

    items: {
	xtype: 'tabpanel',
	bodyPadding: 10,
	items: [
	    {
		title: gettext('General'),
		xtype: 'inputpanel',
		column1: [
		    {
			xtype: 'pmxDisplayEditField',
			cbind: {
			    editable: '{isCreate}',
			},
			name: 'name',
			allowBlank: false,
			fieldLabel: gettext('Name'),
		    },
		    {
			xtype: 'pmxDisplayEditField',
			cbind: {
			    editable: '{isCreate}',
			},
			name: 'path',
			allowBlank: false,
			fieldLabel: gettext('Backing Path'),
			emptyText: gettext('An absolute path'),
		    },
		],
		column2: [
		    {
			xtype: 'pbsCalendarEvent',
			name: 'gc-schedule',
			fieldLabel: gettext("GC Schedule"),
			emptyText: gettext('none'),
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
		    },
		    {
			xtype: 'pbsCalendarEvent',
			name: 'prune-schedule',
			fieldLabel: gettext("Prune Schedule"),
			emptyText: gettext('none'),
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
		    },
		],
		columnB: [
		    {
			xtype: 'textfield',
			name: 'comment',
			fieldLabel: gettext('Comment'),
		    },
		],
	    },
	    {
		title: gettext('Prune Options'),
		xtype: 'inputpanel',
		column1: [
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Last'),
			name: 'keep-last',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Daily'),
			name: 'keep-daily',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Monthly'),
			name: 'keep-monthly',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		],
		column2: [
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Hourly'),
			name: 'keep-hourly',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Weekly'),
			name: 'keep-weekly',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		    {
			xtype: 'proxmoxintegerfield',
			fieldLabel: gettext('Keep Yearly'),
			name: 'keep-yearly',
			cbind: {
			    deleteEmpty: '{!isCreate}',
			},
			minValue: 1,
			allowBlank: true,
		    },
		],
	    },
	],
    },
});
Ext.define('PBS.window.CreateDirectory', {
    extend: 'Proxmox.window.Edit',
    xtype: 'pbsCreateDirectory',

    subject: Proxmox.Utils.directoryText,
    showProgress: true,
    isCreate: true,
    url: '/nodes/localhost/disks/directory',
    method: 'POST',

    items: [
	{
	    xtype: 'pmxDiskSelector',
	    name: 'disk',
	    valueField: 'name',
	    typeProperty: 'usage-type',
	    nodename: 'localhost',
	    diskType: 'unused',
	    fieldLabel: gettext('Disk'),
	    allowBlank: false,
	},
	{
	    xtype: 'proxmoxKVComboBox',
	    comboItems: [
		['ext4', 'ext4'],
		['xfs', 'xfs'],
	    ],
	    fieldLabel: gettext('Filesystem'),
	    name: 'filesystem',
	    value: '',
	    allowBlank: false,
	},
	{
	    xtype: 'proxmoxtextfield',
	    name: 'name',
	    fieldLabel: gettext('Name'),
	    allowBlank: false,
	},
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'add-datastore',
	    fieldLabel: gettext('Add as Datastore'),
	    value: '1',
	},
    ],
});

Ext.define('PBS.window.CreateZFS', {
    extend: 'Proxmox.window.Edit',
    xtype: 'pbsCreateZFS',

    subject: 'ZFS',

    showProgress: true,

    onlineHelp: 'chapter_zfs',

    width: 800,

    url: '/nodes/localhost/disks/zfs',
    method: 'POST',
    items: [
	{
	    xtype: 'inputpanel',
	    onGetValues: function(values) {
		return values;
	    },
	    column1: [
		{
		    xtype: 'proxmoxtextfield',
		    name: 'name',
		    fieldLabel: gettext('Name'),
		    minLength: 3,
		    allowBlank: false,
		},
		{
		    xtype: 'proxmoxcheckbox',
		    name: 'add-datastore',
		    fieldLabel: gettext('Add as Datastore'),
		    value: '1'
		}
	    ],
	    column2: [
		{
		    xtype: 'proxmoxKVComboBox',
		    fieldLabel: gettext('RAID Level'),
		    name: 'raidlevel',
		    value: 'single',
		    comboItems: [
			['single', gettext('Single Disk')],
			['mirror', 'Mirror'],
			['raid10', 'RAID10'],
			['raidz', 'RAIDZ'],
			['raidz2', 'RAIDZ2'],
			['raidz3', 'RAIDZ3']
		    ]
		},
		{
		    xtype: 'proxmoxKVComboBox',
		    fieldLabel: gettext('Compression'),
		    name: 'compression',
		    value: 'on',
		    comboItems: [
			['on', 'on'],
			['off', 'off'],
			['gzip', 'gzip'],
			['lz4', 'lz4'],
			['lzjb', 'lzjb'],
			['zle', 'zle']
		    ]
		},
		{
		    xtype: 'proxmoxintegerfield',
		    fieldLabel: gettext('ashift'),
		    minValue: 9,
		    maxValue: 16,
		    value: '12',
		    name: 'ashift'
		}
	    ],
	    columnB: [
		{
		    xtype: 'pmxMultiDiskSelector',
		    name: 'devices',
		    nodename: 'localhost',
		    typeParameter: 'usage-type',
		    valueField: 'name',
		    height: 200,
		    emptyText: gettext('No Disks unused'),
		}
	    ]
	},
	{
	    xtype: 'displayfield',
	    padding: '5 0 0 0',
	    userCls: 'pmx-hint',
	    value: 'Note: ZFS is not compatible with disks backed by a hardware ' +
	    'RAID controller. For details see ' +
	    '<a target="_blank" href="' + Proxmox.Utils.get_help_link('chapter_zfs') + '">the reference documentation</a>.',
	},
    ],
});
Ext.define('pbs-file-tree', {
    extend: 'Ext.data.Model',

    fields: [ 'filepath', 'text', 'type', 'size',
	{
	    name: 'mtime',
	    type: 'date',
	    dateFormat: 'timestamp',
	},
	{
	    name: 'iconCls',
	    calculate: function(data) {
		let icon = 'file-o';
		switch (data.type) {
		    case 'b': // block device
			icon = 'cube';
			break;
		    case 'c': // char device
			icon = 'tty';
			break;
		    case 'd':
			icon = data.expanded ? 'folder-open-o' : 'folder-o';
			break;
		    case 'f': //regular file
			icon = 'file-text-o';
			break;
		    case 'h': // hardlink
			icon = 'file-o';
			break;
		    case 'l': // softlink
			icon = 'link';
			break;
		    case 'p': // pipe/fifo
			icon = 'exchange';
			break;
		    case 's': // socket
			icon = 'plug';
			break;
		    default:
			icon = 'file-o';
			break;
		}

		return `fa fa-${icon}`;
	    },
	}
    ],
    idProperty: 'filepath',
});

Ext.define("PBS.window.FileBrowser", {
    extend: "Ext.window.Window",

    width: 800,
    height: 600,

    modal: true,

    controller: {
	xclass: 'Ext.app.ViewController',

	buildUrl: function(baseurl, params) {
	    let url = new URL(baseurl, window.location.origin);
	    for (const [key, value] of Object.entries(params)) {
		url.searchParams.append(key, value);
	    }

	    return url.href;
	},

	downloadFile: function() {
	    let me = this;
	    let view = me.getView();
	    let tree = me.lookup('tree');
	    let selection = tree.getSelection();
	    if (!selection || selection.length < 1) return;

	    let data = selection[0].data;

	    let atag = document.createElement('a');

	    atag.download = data.text;
	    let params = {
		'backup-id': view['backup-id'],
		'backup-type': view['backup-type'],
		'backup-time': view['backup-time'],
	    };
	    params['filepath'] = data.filepath;
	    atag.download = data.text;
	    atag.href = me.buildUrl(`/api2/json/admin/datastore/${view.datastore}/pxar-file-download`, params);
	    atag.click();
	},

	fileChanged: function() {
	    let me = this;
	    let view = me.getView();
	    let tree = me.lookup('tree');
	    let selection = tree.getSelection();
	    if (!selection || selection.length < 1) return;

	    let data = selection[0].data;

	    let canDownload = false;
	    switch (data.type) {
		case 'h':
		case 'f':
		    canDownload = true;
		    break;
		default: break;
	    }

	    me.lookup('downloadBtn').setDisabled(!canDownload);
	},

	init: function(view) {
	    let me = this;
	    let tree = me.lookup('tree');

	    if (!view['backup-id']) {
		throw "no backup-id given";
	    }

	    if (!view['backup-type']) {
		throw "no backup-id given";
	    }

	    if (!view['backup-time']) {
		throw "no backup-id given";
	    }

	    if (!view.datastore) {
		throw "no datastore given";
	    }

	    let store = tree.getStore();
	    let proxy = store.getProxy();

	    Proxmox.Utils.monStoreErrors(tree, store, true);
	    proxy.setUrl(`/api2/json/admin/datastore/${view.datastore}/catalog`);
	    proxy.setExtraParams({
		'backup-id': view['backup-id'],
		'backup-type': view['backup-type'],
		'backup-time': view['backup-time'],
	    });
	    store.load(() => {
		let root = store.getRoot();
		root.expand(); // always expand invisible root node
		if (root.childNodes.length === 1) {
		    root.firstChild.expand();
		}
	    });
	},

	control: {
	    'treepanel': {
		selectionchange: 'fileChanged',
	    },
	},
    },

    layout: 'fit',
    items: [
	{
	    xtype: 'treepanel',
	    scrollable: true,
	    rootVisible: false,
	    reference: 'tree',
	    store: {
		autoLoad: false,
		model: 'pbs-file-tree',
		nodeParam: 'filepath',
		sorters: 'text',
		proxy: {
		    appendId: false,
		    type: 'proxmox',
		},
	    },

	    columns: [
		{
		    text: gettext('Name'),
		    xtype: 'treecolumn',
		    flex: 1,
		    dataIndex: 'text',
		    renderer: Ext.String.htmlEncode,
		},
		{
		    text: gettext('Size'),
		    dataIndex: 'size',
		    renderer: value => value === undefined ? '' : Proxmox.Utils.format_size(value),
		    sorter: {
			sorterFn: function(a, b) {
			    let asize = a.data.size || 0;
			    let bsize = b.data.size || 0;

			    return asize - bsize;
			},
		    }
		},
		{
		    text: gettext('Modified'),
		    dataIndex: 'mtime',
		    minWidth: 200,
		},
		{
		    text: gettext('Type'),
		    dataIndex: 'type',
		    renderer: function(value) {
			switch (value) {
			    case 'b': return gettext('Block Device');
			    case 'c': return gettext('Character Device');
			    case 'd': return gettext('Directory');
			    case 'f': return gettext('File');
			    case 'h': return gettext('Hardlink');
			    case 'l': return gettext('Softlink');
			    case 'p': return gettext('Pipe/Fifo');
			    case 's': return gettext('Socket');
			    default: return Proxmox.Utils.unknownText;
			}
		    }
		},
	    ]
	}
    ],

    buttons: [
	{
	    text: gettext('Download'),
	    handler: 'downloadFile',
	    reference: 'downloadBtn',
	    disabled: true,
	}
    ],
});
Ext.define('PBS.window.BackupFileDownloader', {
    extend: 'Ext.window.Window',
    alias: 'widget.pbsBackupFileDownloader',

    title: gettext('Download Files'),
    bodyPadding: 10,

    width: 400,
    modal: true,
    resizable: false,

    layout: {
	type: 'vbox',
	align: 'stretch',
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	buildUrl: function(baseurl, params) {
	    let url = new URL(baseurl, window.location.origin);
	    for (const [key, value] of Object.entries(params)) {
		url.searchParams.append(key, value);
	    }

	    return url.href;
	},

	downloadFile: function() {
	    let me = this;
	    let view = me.getView();
	    let combo = me.lookup('file');
	    let file = combo.getValue();

	    let idx = file.lastIndexOf('.');
	    let filename = file.slice(0, idx);
	    let atag = document.createElement('a');
	    let params = view.params;
	    params['file-name'] = file;
	    atag.download = filename;
	    atag.href = me.buildUrl(`${view.baseurl}/download-decoded`, params);
	    atag.click();
	},

	changeFile: function(comob, value) {
	    let me = this;
	    let combo = me.lookup('file');
	    let rec = combo.getStore().findRecord('filename', value, 0, false, true, true);
	    let canDownload = rec.data['crypt-mode'] !== 'encrypt';
	    me.lookup('encryptedHint').setVisible(!canDownload);
	    me.lookup('signedHint').setVisible(rec.data['crypt-mode'] === 'sign-only');
	    me.lookup('downloadBtn').setDisabled(!canDownload);
	},

	init: function(view) {
	    let me = this;
	    if (!view.baseurl) {
		throw "no baseurl given";
	    }

	    if (!view.params) {
		throw "no params given";
	    }

	    if (!view.files) {
		throw "no files given";
	    }

	    me.lookup('file').getStore().loadData(view.files, false);
	},

	control: {
	    'proxmoxComboGrid': {
		change: 'changeFile'
	    },
	    'button': {
		click: 'downloadFile',
	    }
	},
    },

    items: [
	{
	    xtype: 'proxmoxComboGrid',
	    valueField: 'filename',
	    allowBlank: false,
	    displayField: 'filename',
	    reference: 'file',
	    emptyText: gettext('No file selected'),
	    fieldLabel: gettext('File'),
	    store: {
		fields: ['filename', 'size', 'crypt-mode',],
		idProperty: ['filename'],
	    },
	    listConfig: {
		emptyText: gettext('No Data'),
		columns: [
		    {
			text: gettext('Filename'),
			dataIndex: 'filename',
			renderer: Ext.String.htmlEncode,
			flex: 1,
		    },
		    {
			text: gettext('Size'),
			dataIndex: 'size',
			renderer: val => val === undefined ? '' : Proxmox.Utils.format_size(val),
		    },
		    {
			text: gettext('Encrypted'),
			dataIndex: 'crypt-mode',
			renderer: function(value) {
			    let mode = -1;
			    if (value !== undefined) {
				mode = PBS.Utils.cryptmap.indexOf(value);
			    }
			    return PBS.Utils.cryptText[mode] || Proxmox.Utils.unknownText;
			}
		    },
		],
	    },
	},
	{
	    xtype: 'displayfield',
	    userCls: 'pmx-hint',
	    reference: 'signedHint',
	    hidden: true,
	    value: gettext('Note: Signatures of signed files will not be verified on the server. Please use the client to do this.'),
	},
	{
	    xtype: 'displayfield',
	    userCls: 'pmx-hint',
	    reference: 'encryptedHint',
	    hidden: true,
	    value: gettext('Encrypted Files cannot be decoded on the server directly. Please use the client where the decryption key is located.'),
	}
    ],

    buttons: [
	{
	    text: gettext('Download'),
	    reference: 'downloadBtn',
	    disabled: true,
	},
    ],
});
Ext.define('pbs-datastore-statistics', {
    extend: 'Ext.data.Model',

    fields: [
	'store', 'total', 'used', 'avail', 'estimated-full-date',
	{
	    name: 'history',
	    convert: function(values) {
		let last = null;
		return values.map(v => {
		    if (v !== undefined && v !== null) {
			last = v;
		    }
		    return last;
		});
	    }
	},
	{
	    name: 'usage',
	    calculate: function(data) {
		let used = data.used || 0;
		let total = data.total || 0;
		if (total > 0) {
		    return used/total;
		} else {
		    return -1;
		}
	    }
	},
    ],

    proxy: {
        type: 'proxmox',
	url: "/api2/json/status/datastore-usage"
    },
    idProperty: 'store',
});

Ext.define('PBS.DatastoreStatistics', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.pbsDatastoresStatistics',

    title: gettext('Datastore Usage'),

    emptyText: gettext('No Data'),

    controller: {
	xclass: 'Ext.app.ViewController',

	render_estimate: function(value) {
	    if (!value) {
		return gettext('Not enough data');
	    }

	    let now = new Date();
	    let estimate = new Date(value*1000);

	    let timespan = (estimate - now)/1000;

	    if (+estimate <= +now || isNaN(timespan)) {
		return gettext('Never');
	    }

	    let duration = Proxmox.Utils.format_duration_human(timespan);
	    return Ext.String.format(gettext("in {0}"), duration);
	},

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.getStore().rstore);
	},
    },

    columns: [
	{
	    text: gettext('Name'),
	    dataIndex: 'store',
	    sortable: true,
	},
	{
	    text: gettext('Size'),
	    dataIndex: 'total',
	    sortable: true,
	    width: 90,
	    renderer: Proxmox.Utils.format_size,
	},
	{
	    text: gettext('Used'),
	    dataIndex: 'used',
	    sortable: true,
	    width: 90,
	    renderer: Proxmox.Utils.format_size,
	},
	{
	    text: gettext('Usage %'),
	    dataIndex: 'usage',
	    sortable: true,
	    xtype: 'widgetcolumn',
	    widget: {
		xtype: 'progressbarwidget',
		bind: '{record.usage}',
		textTpl: [
		    '<tpl if="value &gt;= 0">',
		    '{value:number("0.00")*100}%',
		    '<tpl else>',
		    Proxmox.Utils.unknownText,
		    '</tpl>',
		],
	    },
	},
	{
	    text: gettext('Estimated Full'),
	    dataIndex: 'estimated-full-date',
	    sortable: true,
	    renderer: 'render_estimate',
	    flex: 1,
	    minWidth: 130,
	    maxWidth: 200,
	},
	{
	    text: gettext("History (last Month)"),
	    width: 100,
	    xtype: 'widgetcolumn',
	    dataIndex: 'history',
	    flex: 1,
	    widget: {
		xtype: 'sparklineline',
		bind: '{record.history}',
		spotRadius: 0,
		fillColor: '#ddd',
		lineColor: '#555',
		lineWidth: 0,
		chartRangeMin: 0,
		chartRangeMax: 1,
		tipTpl: '{y:number("0.00")*100}%'
	    }
	},
    ],

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: 'store',
	rstore: {
	    type: 'update',
	    storeid: 'pbs-datastore-statistics',
	    model: 'pbs-datastore-statistics',
	    autoStart: true,
	    interval: 30000,
	},
    },

})
Ext.define('PBS.LongestTasks', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.pbsLongestTasks',

    title: gettext('Longest Tasks (last Month)'),

    hideHeaders: true,
    rowLines: false,

    emptyText: gettext('No Tasks'),

    controller: {
	xclass: 'Ext.app.ViewController',

	openTask: function(record) {
	    let me = this;
	    let view = me.getView();
	    Ext.create('Proxmox.window.TaskViewer', {
		upid: record.data.upid,
		endtime: record.data.endtime,
	    }).show();
	},

	openTaskItemDblClick: function(grid, record) {
	    this.openTask(record);
	},

	openTaskActionColumn: function(grid, rowIndex) {
	    this.openTask(grid.getStore().getAt(rowIndex));
	},

	render_status: function(value) {
	    let cls = 'times-circle critical';
	    if (value === 'OK') {
		cls = 'check-circle good';
	    } else if (value.startsWith('WARNINGS:')) {
		cls = 'exclamation-circle warning';
	    } else if (value === 'unknown') {
		cls = 'question-circle faded';
	    }

	    return `<i class="fa fa-${cls}"></i>`;
	},
    },

    updateTasks: function(data) {
	let me = this;
	me.getStore().setData(data);
    },

    listeners: {
	itemdblclick: 'openTaskItemDblClick',
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	autoDestroyRstore: true,
	sorters: [
	    {
		property: 'duration',
		direction: 'DESC',
	    },
	    {
		property: 'upid',
		direction: 'ASC',
	    },
	],
	rstore: {
	    storeid: 'proxmox-tasks-dash',
	    type: 'store',
	    model: 'proxmox-tasks',
	    proxy: {
		type: 'memory',
	    }
	},
    },

    columns: [
	{
	    text: gettext('Task'),
	    dataIndex: 'upid',
	    renderer: Proxmox.Utils.render_upid,
	    flex: 1,
	},
	{
	    text: gettext('Duration'),
	    dataIndex: 'duration',
	    renderer: Proxmox.Utils.format_duration_human,
	},
	{
	    text: gettext('Status'),
	    align: 'center',
	    width: 40,
	    dataIndex: 'status',
	    renderer: 'render_status',
	},
	{
	    xtype: 'actioncolumn',
	    width: 40,
	    items: [
		{
		    iconCls: 'fa fa-chevron-right',
		    tooltip: gettext('Open Task'),
		    handler: 'openTaskActionColumn',
		},
	    ],
	},
    ],
});
Ext.define('PBS.RunningTasks', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.pbsRunningTasks',

    title: gettext('Running Tasks'),
    emptyText: gettext('No running tasks'),

    hideHeaders: true,
    rowLines: false,

    controller: {
	xclass: 'Ext.app.ViewController',

	openTask: function(record) {
	    let me = this;
	    let view = me.getView();
	    Ext.create('Proxmox.window.TaskViewer', {
		upid: record.data.upid,
		endtime: record.data.endtime,
	    }).show();

	    view.fireEvent('taskopened', view, record.data.upid);
	},

	openTaskItemDblClick: function(grid, record) {
	    this.openTask(record);
	},

	openTaskActionColumn: function(grid, rowIndex) {
	    this.openTask(grid.getStore().getAt(rowIndex));
	},

	render_status: function(value) {
	    let cls = 'times-circle critical';
	    if (value === 'OK') {
		cls = 'check-circle good';
	    } else if (value.startsWith('WARNINGS:')) {
		cls = 'exclamation-circle warning';
	    } else if (value === 'unknown') {
		cls = 'question-circle faded';
	    }

	    return `<i class="fa fa-${cls}"></i>`;
	},
    },

    updateTasks: function(data) {
	let me = this;
	me.getStore().setData(data);
    },

    listeners: {
	itemdblclick: 'openTaskItemDblClick',
    },

    store: {
	type: 'diff',
	autoDestroy: true,
	sorters: 'starttime',
	rstore: PBS.data.RunningTasksStore,
    },

    columns: [
	{
	    text: 'Task',
	    dataIndex: 'upid',
	    renderer: Proxmox.Utils.render_upid,
	    flex: 2,
	},
	{
	    text: 'Starttime',
	    dataIndex: 'starttime',
	    renderer: function(value) {
		return Ext.Date.format(value, "Y-m-d H:i:s");
	    },
	    flex: 1,
	},
	{
	    text: 'Duration',
	    dataIndex: 'duration',
	    renderer: function(value, md, record) {
		return Proxmox.Utils.format_duration_human((Date.now() - record.data.starttime)/1000);
	    }
	},
	{
	    xtype: 'actioncolumn',
	    width: 40,
	    items: [
		{
		    iconCls: 'fa fa-chevron-right',
		    tooltip: gettext('Open Task'),
		    handler: 'openTaskActionColumn',
		},
	    ],
	},
    ],
});

Ext.define('PBS.TaskSummary', {
    extend: 'Ext.panel.Panel',
    alias: 'widget.pbsTaskSummary',

    title: gettext('Task Summary (last Month)'),

    controller: {
	xclass: 'Ext.app.ViewController',

	render_count: function(value, md, record, rowindex, colindex) {
	    let cls = 'question';
	    let color = 'faded';
	    switch (colindex) {
		case 1:
		    cls = "times-circle";
		    color = "critical";
		    break;
		case 2:
		    cls = "exclamation-circle";
		    color = "warning";
		    break;
		case 3:
		    cls = "check-circle";
		    color = "good";
		    break;
		default: break;
	    }

	    if (value < 1) {
		color = "faded";
	    }
	    cls += " " + color;
	    return `<i class="fa fa-${cls}"></i> ${value}`;
	},
    },

    updateTasks: function(data) {
	let me = this;
	data.backup.type = gettext('Backups');
	data.prune.type = gettext('Prunes');
	data.garbage_collection.type = gettext('Garbage collections');
	data.sync.type = gettext('Syncs');
	me.lookup('grid').getStore().setData([
	    data.backup,
	    data.prune,
	    data.garbage_collection,
	    data.sync,
	]);
    },

    layout: 'fit',
    bodyPadding: 15,
    minHeight: 166,

    // we have to wrap the grid in a panel to get the padding right
    items: [
	{
	    xtype: 'grid',
	    reference: 'grid',
	    hideHeaders: true,
	    border: false,
	    bodyBorder: false,
	    rowLines: false,
	    viewConfig: {
		stripeRows: false,
		trackOver: false,
	    },
	    scrollable: false,
	    disableSelection: true,

	    store: {
		data: []
	    },

	    columns: [
		{
		    dataIndex: 'type',
		    flex: 1,
		},
		{
		    dataIndex: 'error',
		    renderer: 'render_count',
		},
		{
		    dataIndex: 'warning',
		    renderer: 'render_count',
		},
		{
		    dataIndex: 'ok',
		    renderer: 'render_count',
		},
	    ],
	}
    ],

});
/*global Proxmox */
Ext.ns('PBS');

console.log("Starting Backup Server GUI");

Ext.define('PBS.Utils', {
    singleton: true,

    updateLoginData: function(data) {

	Proxmox.Utils.setAuthData(data);
    },

    dataStorePrefix: 'DataStore-',

    cryptmap: [
	'none',
	'mixed',
	'sign-only',
	'encrypt',
    ],

    cryptText: [
	Proxmox.Utils.noText,
	gettext('Mixed'),
	gettext('Signed'),
	gettext('Encrypted'),
    ],

    cryptIconCls: [
	'',
	'',
	'certificate',
	'lock',
    ],

    calculateCryptMode: function(data) {
	let mixed = data.mixed;
	let encrypted = data.encrypt;
	let signed = data['sign-only'];
	let files = data.count;
	if (mixed > 0) {
	    return PBS.Utils.cryptmap.indexOf('mixed');
	} else if (files === encrypted) {
	    return PBS.Utils.cryptmap.indexOf('encrypt');
	} else if (files === signed) {
	    return PBS.Utils.cryptmap.indexOf('sign-only');
	} else if ((signed+encrypted) === 0) {
	    return PBS.Utils.cryptmap.indexOf('none');
	} else {
	    return PBS.Utils.cryptmap.indexOf('mixed');
	}
    },

    getDataStoreFromPath: function(path) {
	return path.slice(PBS.Utils.dataStorePrefix.length);
    },

    isDataStorePath: function(path) {
	return path.indexOf(PBS.Utils.dataStorePrefix) === 0;
    },

    render_datetime_utc: function(datetime) {
	let pad = (number) => number < 10 ? '0' + number : number;
	return datetime.getUTCFullYear() +
	    '-' + pad(datetime.getUTCMonth() + 1) +
	    '-' + pad(datetime.getUTCDate()) +
	    'T' + pad(datetime.getUTCHours()) +
	    ':' + pad(datetime.getUTCMinutes()) +
	    ':' + pad(datetime.getUTCSeconds()) +
	    'Z';
    },

    render_datastore_worker_id: function(id, what) {
	const res = id.match(/^(\S+?)_(\S+?)_(\S+?)(_(.+))?$/);
	if (res) {
	    let datastore = res[1], type = res[2], id = res[3];
	    if (res[4] !== undefined) {
		let datetime = Ext.Date.parse(parseInt(res[5], 16), 'U');
		let utctime = PBS.Utils.render_datetime_utc(datetime);
		return `Datastore ${datastore} ${what} ${type}/${id}/${utctime}`;
	    } else {
		return `Datastore ${datastore} ${what} ${type}/${id}`;
	    }
	}
	return `Datastore ${what} ${id}`;
    },

    constructor: function() {
	var me = this;

	// do whatever you want here
	Proxmox.Utils.override_task_descriptions({
	    garbage_collection: ['Datastore', gettext('Garbage collect') ],
	    sync: ['Datastore', gettext('Remote Sync') ],
	    syncjob: [gettext('Sync Job'), gettext('Remote Sync') ],
	    prune: (type, id) => {
		return PBS.Utils.render_datastore_worker_id(id, gettext('Prune'));
	    },
	    verify: (type, id) => {
		return PBS.Utils.render_datastore_worker_id(id, gettext('Verify'));
	    },
	    backup: (type, id) => {
		return PBS.Utils.render_datastore_worker_id(id, gettext('Backup'));
	    },
	    reader: (type, id) => {
		return PBS.Utils.render_datastore_worker_id(id, gettext('Read objects'));
	    },
	});
    }
});
Ext.define('PBS.admin.ZFSList', {
    extend: 'Ext.grid.Panel',
    xtype: 'pbsZFSList',

    stateful: true,
    stateId: 'grid-node-zfs',

    controller: {
	xclass: 'Ext.app.ViewController',

	openCreateWindow: function() {
	    let me = this;
	    Ext.create('PBS.window.CreateZFS', {
		nodename: me.nodename,
		listeners: {
		    destroy: function() { me.reload(); },
		}
	    }).show();
	},

	openDetailWindow: function() {
	    let me = this;
	    let view = me.getView();
	    let selection = view.getSelection();
	    if (!selection || selection.length < 1) return;

	    let rec = selection[0];
	    let zpool = rec.get('name');

	    Ext.create('Proxmox.window.ZFSDetail', {
		zpool,
		nodename: view.nodename,
	    }).show();
	},

	reload: function() {
	    let me = this;
	    let view = me.getView();
	    let store = view.getStore();
	    store.load();
	    store.sort();
	},

	init: function(view) {
	    let me = this;

	    if (!view.nodename) {
		throw "no nodename given";
	    }

	    let url = `/api2/json/nodes/${view.nodename}/disks/zfs`;
	    view.getStore().getProxy().setUrl(url)

	    Proxmox.Utils.monStoreErrors(view, view.getStore(), true);

	    me.reload();
	},
    },

    columns: [
	{
	    text: gettext('Name'),
	    dataIndex: 'name',
	    flex: 1
	},
	{
	    header: gettext('Size'),
	    renderer: Proxmox.Utils.format_size,
	    dataIndex: 'size'
	},
	{
	    header: gettext('Free'),
	    renderer: Proxmox.Utils.format_size,
	    dataIndex: 'free'
	},
	{
	    header: gettext('Allocated'),
	    renderer: Proxmox.Utils.format_size,
	    dataIndex: 'alloc'
	},
	{
	    header: gettext('Fragmentation'),
	    renderer: function(value) {
		return value.toString() + '%';
	    },
	    dataIndex: 'frag'
	},
	{
	    header: gettext('Health'),
	    renderer: Proxmox.Utils.render_zfs_health,
	    dataIndex: 'health'
	},
	{
	    header: gettext('Deduplication'),
	    hidden: true,
	    renderer: function(value) {
		return value.toFixed(2).toString() + 'x';
	    },
	    dataIndex: 'dedup'
	}
    ],

    rootVisible: false,
    useArrows: true,

    tbar: [
	{
	    text: gettext('Reload'),
	    iconCls: 'fa fa-refresh',
	    handler: 'reload',
	},
	{
	    text: gettext('Create') + ': ZFS',
	    handler: 'openCreateWindow',
	},
	{
	    text: gettext('Detail'),
	    xtype: 'proxmoxButton',
	    disabled: true,
	    handler: 'openDetailWindow',
	}
    ],

    listeners: {
	itemdblclick: 'openDetailWindow',
    },

    store: {
	fields: ['name', 'size', 'free', 'alloc', 'dedup', 'frag', 'health'],
	proxy: {
	    type: 'proxmox',
	},
	sorters: 'name'
    },
});

Ext.define('PBS.admin.Directorylist', {
    extend: 'Ext.grid.Panel',
    xtype: 'pbsDirectoryList',

    stateful: true,
    stateId: 'grid-node-directory',

    emptyText: gettext('No Mount-Units found'),

    controller: {
	xclass: 'Ext.app.ViewController',

	createDirectory: function() {
	    let me = this;
	    Ext.create('PBS.window.CreateDirectory', {
		listeners: {
		    destroy: function() {
			me.reload();
		    },
		},
	    }).show();
	},

	reload: function() {
	    let me = this;
	    let store = me.getView().getStore();
	    store.load();
	    store.sort();
	},

	init: function(view) {
	    let me = this;
	    Proxmox.Utils.monStoreErrors(view, view.getStore(), true);
	    me.reload();
	},
    },


    rootVisible: false,
    useArrows: true,

    tbar: [
	{
	    text: gettext('Reload'),
	    iconCls: 'fa fa-refresh',
	    handler: 'reload',
	},
	{
	    text: gettext('Create') + ': Directory',
	    handler: 'createDirectory',
	},
    ],

    columns: [
	{
	    text: gettext('Path'),
	    dataIndex: 'path',
	    flex: 1,
	},
	{
	    header: gettext('Device'),
	    flex: 1,
	    dataIndex: 'device',
	},
	{
	    header: gettext('Filesystem'),
	    width: 100,
	    dataIndex: 'filesystem',
	},
	{
	    header: gettext('Options'),
	    width: 100,
	    dataIndex: 'options',
	},
	{
	    header: gettext('Unit File'),
	    hidden: true,
	    dataIndex: 'unitfile',
	},
    ],

    store: {
	fields: ['path', 'device', 'filesystem', 'options', 'unitfile'],
	proxy: {
	    type: 'proxmox',
	    url: '/api2/json/nodes/localhost/disks/directory',
	},
	sorters: 'path',
    },
});
Ext.define('PBS.LoginView', {
    extend: 'Ext.container.Container',
    xtype: 'loginview',

    controller: {
	xclass: 'Ext.app.ViewController',

	submitForm: function() {
	    var me = this;
	    var view = me.getView();
	    var loginForm = me.lookupReference('loginForm');
	    var unField = me.lookupReference('usernameField');
	    var saveunField = me.lookupReference('saveunField');

	    if (!loginForm.isValid()) {
		return;
	    }

	    let params = loginForm.getValues();

	    params.username = params.username + '@' + params.realm;
	    delete(params.realm);

	    if (loginForm.isVisible()) {
		loginForm.mask(gettext('Please wait...'), 'x-mask-loading');
	    }

	    // set or clear username
	    var sp = Ext.state.Manager.getProvider();
	    if (saveunField.getValue() === true) {
		sp.set(unField.getStateId(), unField.getValue());
	    } else {
		sp.clear(unField.getStateId());
	    }
	    sp.set(saveunField.getStateId(), saveunField.getValue());

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/access/ticket',
		params: params,
		method: 'POST',
		success: function(resp, opts) {
		    // save login data and create cookie
		    PBS.Utils.updateLoginData(resp.result.data);
		    PBS.app.changeView('mainview');
		},
		failure: function(resp, opts) {
		    Proxmox.Utils.authClear();
		    loginForm.unmask();
		    Ext.MessageBox.alert(
			gettext('Error'),
			gettext('Login failed. Please try again')
		    );
		}
	    });
	},

	control: {
	    'field[name=username]': {
		specialkey: function(f, e) {
		    if (e.getKey() === e.ENTER) {
			var pf = this.lookupReference('passwordField');
			if (!pf.getValue()) {
			    pf.focus(false);
			}
		    }
		}
	    },
	    'field[name=lang]': {
		change: function(f, value) {
		    var dt = Ext.Date.add(new Date(), Ext.Date.YEAR, 10);
		    Ext.util.Cookies.set('PBSLangCookie', value, dt);
		    this.getView().mask(gettext('Please wait...'), 'x-mask-loading');
		    window.location.reload();
		}
	    },
	    'button[reference=loginButton]': {
		click: 'submitForm'
	    },
	    'window[reference=loginwindow]': {
		show: function() {
		    var sp = Ext.state.Manager.getProvider();
		    var checkboxField = this.lookupReference('saveunField');
		    var unField = this.lookupReference('usernameField');

		    var checked = sp.get(checkboxField.getStateId());
		    checkboxField.setValue(checked);

		    if(checked === true) {
			var username = sp.get(unField.getStateId());
			unField.setValue(username);
			var pwField = this.lookupReference('passwordField');
			pwField.focus();
		    }
		}
	    }
	}
    },

    plugins: 'viewport',

    layout: {
	type: 'border'
    },

    items: [
	{
	    region: 'north',
	    xtype: 'container',
	    layout: {
		type: 'hbox',
		align: 'middle'
	    },
	    margin: '2 5 2 5',
	    height: 38,
	    items: [
		{
		    xtype: 'proxmoxlogo',
		    prefix: '',
		},
		{
		    xtype: 'versioninfo',
		    makeApiCall: false
		}
	    ]
	},
	{
	    region: 'center'
	},
	{
	    xtype: 'window',
	    closable: false,
	    resizable: false,
	    reference: 'loginwindow',
	    autoShow: true,
	    modal: true,
	    width: 400,

	    defaultFocus: 'usernameField',

	    layout: {
		type: 'auto'
	    },

	    title: gettext('Proxmox Backup Server Login'),

	    items: [
		{
		    xtype: 'form',
		    layout: {
			type: 'form'
		    },
		    defaultButton: 'loginButton',
		    url: '/api2/extjs/access/ticket',
		    reference: 'loginForm',

		    fieldDefaults: {
			labelAlign: 'right',
			allowBlank: false
		    },

		    items: [
			{
			    xtype: 'textfield',
			    fieldLabel: gettext('User name'),
			    name: 'username',
			    itemId: 'usernameField',
			    reference: 'usernameField',
			    stateId: 'login-username'
			},
			{
			    xtype: 'textfield',
			    inputType: 'password',
			    fieldLabel: gettext('Password'),
			    name: 'password',
			    itemId: 'passwordField',
			    reference: 'passwordField',
			},
			{
			    xtype: 'pmxRealmComboBox',
			    name: 'realm'
			},
			{
			    xtype: 'proxmoxLanguageSelector',
			    fieldLabel: gettext('Language'),
			    value: Ext.util.Cookies.get('PBSLangCookie') || Proxmox.defaultLang || 'en',
			    name: 'lang',
			    reference: 'langField',
			    submitValue: false
			}
		    ],
		    buttons: [
			{
			    xtype: 'checkbox',
			    fieldLabel: gettext('Save User name'),
			    name: 'saveusername',
			    reference: 'saveunField',
			    stateId: 'login-saveusername',
			    labelWidth: 250,
			    labelAlign: 'right',
			    submitValue: false
			},
			{
			    text: gettext('Login'),
			    reference: 'loginButton',
			    formBind: true
			}
		    ]
		}
	    ]
	}
    ]
});
/*global Proxmox*/
Ext.define('PBS.view.main.VersionInfo',{
    extend: 'Ext.Component',
    xtype: 'versioninfo',

    makeApiCall: true,

    data: {
	version: false
    },

    tpl: [
	'Backup Server',
	'<tpl if="version">',
	' {version}-{release}',
	'</tpl>'
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	if (me.makeApiCall) {
	    Proxmox.Utils.API2Request({
		url: '/version',
		method: 'GET',
		success: function(response) {
		    me.update(response.result.data);
		}
	    });
	}
    }
});
/*global Proxmox*/

Ext.define('PBS.SystemConfiguration', {
    extend: 'Ext.tab.Panel',
    xtype: 'pbsSystemConfiguration',

    title: gettext('Configuration') + ': ' + gettext('System'),
    border: true,
    scrollable: true,
    defaults: { border: false },
    items: [
	{
	    title: gettext('Network/Time'),
	    itemId: 'network',
	    xtype: 'panel',
	    layout: {
		type: 'vbox',
		align: 'stretch',
		multi: true
	    },
	    defaults: {
		collapsible: true,
		animCollapse: false,
		margin: '10 10 0 10'
	    },
	    items: [
		{
		    title: gettext('Time'),
		    xtype: 'proxmoxNodeTimeView',
		    nodename: 'localhost'
		},
		{
		    title: gettext('DNS'),
		    xtype: 'proxmoxNodeDNSView',
		    nodename: 'localhost'
		},
		{
		    flex: 1,
		    minHeight: 200,
		    title: gettext('Network Interfaces'),
		    xtype: 'proxmoxNodeNetworkView',
		    showApplyBtn: true,
		    types: ['bond', 'bridge', 'vlan'],
		    nodename: 'localhost'
		},
	    ]
//	},
//	{
//	    itemId: 'options',
//          title: gettext('Options'),
//	    html: "TESWT"
//	    xtype: 'pbsSystemOptions'
	}
    ],

    initComponent: function() {
	var me = this;

	me.callParent();

	var networktime = me.getComponent('network');
	Ext.Array.forEach(networktime.query(), function(item) {
	    item.relayEvents(networktime, [ 'activate', 'deactivate', 'destroy']);
	});
    }
});


/*global Blob,Proxmox*/
Ext.define('PBS.SubscriptionKeyEdit', {
    extend: 'Proxmox.window.Edit',

    title: gettext('Upload Subscription Key'),
    width: 300,
    autoLoad: true,

    onlineHelp: 'getting_help',

    items: {
	xtype: 'textfield',
	name: 'key',
	value: '',
	fieldLabel: gettext('Subscription Key')
    }
});

Ext.define('PBS.Subscription', {
    extend: 'Proxmox.grid.ObjectGrid',
    xtype: 'pbsSubscription',

    title: gettext('Subscription'),

    border: true,

    onlineHelp: 'getting_help',

    viewConfig: {
	enableTextSelection: true
    },

    initComponent : function() {
	var me = this;

	var reload = function() {
	    me.rstore.load();
	};

	var baseurl = '/nodes/localhost/subscription';

	var render_status = function(value) {

	    var message = me.getObjectValue('message');

	    if (message) {
		return value + ": " + message;
	    }
	    return value;
	};

	var rows = {
	    productname: {
		header: gettext('Type')
	    },
	    key: {
		header: gettext('Subscription Key')
	    },
	    status: {
		header: gettext('Status'),
		renderer: render_status
	    },
	    message: {
		visible: false
	    },
	    serverid: {
		header: gettext('Server ID')
	    },
	    sockets: {
		header: gettext('Sockets')
	    },
	    checktime: {
		header: gettext('Last checked'),
		renderer: Proxmox.Utils.render_timestamp
	    },
	    nextduedate: {
		header: gettext('Next due date')
	    }
	};

	Ext.apply(me, {
	    url: '/api2/json' + baseurl,
	    cwidth1: 170,
	    tbar: [
		{
		    text: gettext('Upload Subscription Key'),
		    handler: function() {
			var win = Ext.create('PBS.SubscriptionKeyEdit', {
			    url: '/api2/extjs/' + baseurl
			});
			win.show();
			win.on('destroy', reload);
		    }
		},
		{
		    text: gettext('Check'),
		    handler: function() {
			Proxmox.Utils.API2Request({
			    params: { force: 1 },
			    url: baseurl,
			    method: 'POST',
			    waitMsgTarget: me,
			    failure: function(response, opts) {
				Ext.Msg.alert(gettext('Error'), response.htmlStatus);
			    },
			    callback: reload
			});
		    }
		}
	    ],
	    rows: rows
	});

	me.callParent();

	reload();
    }
});
Ext.define('pbs-prune-list', {
    extend: 'Ext.data.Model',
    fields: [
	'backup-type',
	'backup-id',
	{
	    name: 'backup-time',
	    type: 'date',
	    dateFormat: 'timestamp'
	},
    ]
});

Ext.define('PBS.DataStorePruneInputPanel', {
    extend: 'Proxmox.panel.InputPanel',
    alias: 'widget.pbsDataStorePruneInputPanel',
    mixins: ['Proxmox.Mixin.CBind'],

    onGetValues: function(values) {
	var me = this;

	values["backup-type"] = me.backup_type;
	values["backup-id"] = me.backup_id;
	return values;
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    if (!view.url) {
		throw "no url specified";
	    }
	    if (!view.backup_type) {
		throw "no backup_type specified";
	    }
	    if (!view.backup_id) {
		throw "no backup_id specified";
	    }

	    this.reload(); // initial load
	},

	reload: function() {
	    var view = this.getView();

	    let params = view.getValues();
	    params["dry-run"] = true;

	    Proxmox.Utils.API2Request({
		url: view.url,
		method: "POST",
		params: params,
		callback: function() {
		    return; // for easy breakpoint setting
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, options) {
		    var data = response.result.data;
		    view.prune_store.setData(data);
		}
	    });
	},

	control: {
	    field: { change: 'reload' }
	}
    },

    column1: [
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-last',
	    allowBlank: true,
	    fieldLabel: gettext('keep-last'),
	    minValue: 1,
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-hourly',
	    allowBlank: true,
	    fieldLabel: gettext('keep-hourly'),
	    minValue: 1,
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-daily',
	    allowBlank: true,
	    fieldLabel: gettext('keep-daily'),
	    minValue: 1,
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-weekly',
	    allowBlank: true,
	    fieldLabel: gettext('keep-weekly'),
	    minValue: 1,
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-monthly',
	    allowBlank: true,
	    fieldLabel: gettext('keep-monthly'),
	    minValue: 1,
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'keep-yearly',
	    allowBlank: true,
	    fieldLabel: gettext('keep-yearly'),
	    minValue: 1,
	}
    ],


    initComponent : function() {
        var me = this;

	me.prune_store = Ext.create('Ext.data.Store', {
	    model: 'pbs-prune-list',
	    sorters: { property: 'backup-time', direction: 'DESC' }
	});

	me.column2 = [
	    {
		xtype: 'grid',
		height: 200,
		store: me.prune_store,
		columns: [
		    {
			header: gettext('Backup Time'),
			sortable: true,
			dataIndex: 'backup-time',
			renderer: function(value, metaData, record) {
			    let text = Ext.Date.format(value, 'Y-m-d H:i:s');
			    if (record.data.keep) {
				return text;
			    } else {
				return '<div style="text-decoration: line-through;">'+ text +'</div>';
			    }
			},
			flex: 1,
		    },
		    {
			text: "keep",
			dataIndex: 'keep'
		    }
		]
	    }
	];

	me.callParent();
    }
});

Ext.define('PBS.DataStorePrune', {
    extend: 'Proxmox.window.Edit',

    method: 'POST',
    submitText: "Prune",

    isCreate: true,

    initComponent : function() {
        var me = this;

	if (!me.datastore) {
	    throw "no datastore specified";
	}
	if (!me.backup_type) {
	    throw "no backup_type specified";
	}
	if (!me.backup_id) {
	    throw "no backup_id specified";
	}

	Ext.apply(me, {
	    url: '/api2/extjs/admin/datastore/' + me.datastore + "/prune",
	    title: "Prune Datastore '" + me.datastore + "'",
	    items: [{
		xtype: 'pbsDataStorePruneInputPanel',
		url: '/api2/extjs/admin/datastore/' + me.datastore + "/prune",
		backup_type: me.backup_type,
		backup_id: me.backup_id
	    }]
	});

	me.callParent();
    }
});
Ext.define('pve-rrd-datastore', {
    extend: 'Ext.data.Model',
    fields: [
	'used',
	'total',
	'read_ios',
	'read_bytes',
	'write_ios',
	'write_bytes',
        'io_ticks',
	{
	    name: 'io_delay', calculate: function(data) {
		let ios = 0;
		if (data.read_ios !== undefined) { ios += data.read_ios; }
		if (data.write_ios !== undefined) { ios += data.write_ios; }
		if (data.io_ticks === undefined) {
		    return undefined;
		} else if (ios === 0) {
		    return 0;
		}
		return (data.io_ticks*1000.0)/ios;
	    }
	},
	{ type: 'date', dateFormat: 'timestamp', name: 'time' }
    ]
});

Ext.define('PBS.DataStoreStatistic', {
    extend: 'Ext.panel.Panel',
    alias: 'widget.pbsDataStoreStatistic',

    title: gettext('Statistics'),

    scrollable: true,

    initComponent: function() {
        var me = this;

	if (!me.datastore) {
	    throw "no datastore specified";
	}

	me.tbar = [ '->', { xtype: 'proxmoxRRDTypeSelector' } ];

	var rrdstore = Ext.create('Proxmox.data.RRDStore', {
	    rrdurl: "/api2/json/admin/datastore/" + me.datastore + "/rrd",
	    model: 'pve-rrd-datastore'
	});

	me.items = {
	    xtype: 'container',
	    itemId: 'itemcontainer',
	    layout: 'column',
	    minWidth: 700,
	    defaults: {
		minHeight: 320,
		padding: 5,
		columnWidth: 1
	    },
	    items: [
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Storage usage (bytes)'),
		    fields: ['total','used'],
		    fieldTitles: [gettext('Total'), gettext('Storage usage')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Transfer Rate (bytes/second)'),
		    fields: ['read_bytes','write_bytes'],
		    fieldTitles: [gettext('Read'), gettext('Write')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Input/Output Operations per Second (IOPS)'),
		    fields: ['read_ios','write_ios'],
		    fieldTitles: [gettext('Read'), gettext('Write')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('IO Delay (ms)'),
		    fields: ['io_delay'],
		    fieldTitles: [gettext('IO Delay')],
		    store: rrdstore
		},
	    ]
	};

	me.listeners = {
	    activate: function() {
		rrdstore.startUpdate();
	    },
	    destroy: function() {
		rrdstore.stopUpdate();
	    },
	};

	me.callParent();
    }

});
Ext.define('pbs-data-store-snapshots', {
    extend: 'Ext.data.Model',
    fields: [
	'backup-type',
	'backup-id',
	{
	    name: 'backup-time',
	    type: 'date',
	    dateFormat: 'timestamp'
	},
	'files',
	'owner',
	{ name: 'size', type: 'int', allowNull: true, },
	{
	    name: 'crypt-mode',
	    type: 'boolean',
	    calculate: function(data) {
		let encrypted = 0;
		let crypt = {
		    none: 0,
		    mixed: 0,
		    'sign-only': 0,
		    encrypt: 0,
		    count: 0,
		};
		let signed = 0;
		data.files.forEach(file => {
		    if (file.filename === 'index.json.blob') return; // is never encrypted
		    let mode = PBS.Utils.cryptmap.indexOf(file['crypt-mode']);
		    if (mode !== -1) {
			crypt[file['crypt-mode']]++;
		    }
		    crypt.count++;
		});

		return PBS.Utils.calculateCryptMode(crypt);
	    }
	}
    ]
});

Ext.define('PBS.DataStoreContent', {
    extend: 'Ext.tree.Panel',
    alias: 'widget.pbsDataStoreContent',

    rootVisible: false,

    title: gettext('Content'),

    controller: {
	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    if (!view.datastore) {
		throw "no datastore specified";
	    }

	    this.store = Ext.create('Ext.data.Store', {
		model: 'pbs-data-store-snapshots',
		groupField: 'backup-group',
	    });
	    this.store.on('load', this.onLoad, this);

	    view.getStore().setSorters([
		'backup-group',
		'text',
		'backup-time'
	    ]);
	    Proxmox.Utils.monStoreErrors(view, this.store);
	    this.reload(); // initial load
	},

	reload: function() {
	    let view = this.getView();

	    if (!view.store || !this.store) {
		console.warn('cannot reload, no store(s)');
		return;
	    }

	    let url = `/api2/json/admin/datastore/${view.datastore}/snapshots`;
	    this.store.setProxy({
		type: 'proxmox',
		timeout: 300*1000, // 5 minutes, we should make that api call faster
		url:  url
	    });

	    this.store.load();
	},

	getRecordGroups: function(records) {
	    let groups = {};

	    for (const item of records) {
		var btype = item.data["backup-type"];
		let group = btype + "/" + item.data["backup-id"];

		if (groups[group] !== undefined) {
		    continue;
		}

		var cls = '';
		if (btype === 'vm') {
		    cls = 'fa-desktop';
		} else if (btype === 'ct') {
		    cls = 'fa-cube';
		} else if (btype === 'host') {
		    cls = 'fa-building';
		} else {
		    console.warn(`got unknown backup-type '${btype}'`);
		    continue; // FIXME: auto render? what do?
		}

		groups[group] = {
		    text: group,
		    leaf: false,
		    iconCls: "fa " + cls,
		    expanded: false,
		    backup_type: item.data["backup-type"],
		    backup_id: item.data["backup-id"],
		    children: []
		};
	    }

	    return groups;
	},

	onLoad: function(store, records, success, operation) {
	    let view = this.getView();

	    if (!success) {
		Proxmox.Utils.setErrorMask(view, Proxmox.Utils.getResponseErrorMessage(operation.getError()));
		return;
	    }

	    let groups = this.getRecordGroups(records);

	    for (const item of records) {
		let group = item.data["backup-type"] + "/" + item.data["backup-id"];
		let children = groups[group].children;

		let data = item.data;

		data.text = group + '/' + PBS.Utils.render_datetime_utc(data["backup-time"]);
		data.leaf = true;
		data.cls = 'no-leaf-icons';

		children.push(data);
	    }

	    let children = [];
	    for (const [_key, group] of Object.entries(groups)) {
		let last_backup = 0;
		let crypt = {
		    none: 0,
		    mixed: 0,
		    'sign-only': 0,
		    encrypt: 0,
		};
		for (const item of group.children) {
		    crypt[PBS.Utils.cryptmap[item['crypt-mode']]]++;
		    if (item["backup-time"] > last_backup && item.size !== null) {
			last_backup = item["backup-time"];
			group["backup-time"] = last_backup;
			group.files = item.files;
			group.size = item.size;
			group.owner = item.owner;
		    }

		}
		group.count = group.children.length;
		crypt.count = group.count;
		group['crypt-mode'] = PBS.Utils.calculateCryptMode(crypt);
		children.push(group);
	    }

	    view.setRootNode({
		expanded: true,
		children: children
	    });
	    Proxmox.Utils.setErrorMask(view, false);
	},

	onPrune: function() {
	    var view = this.getView();

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;
	    if (data.leaf) return;

	    if (!view.datastore) return;

	    let win = Ext.create('PBS.DataStorePrune', {
		datastore: view.datastore,
		backup_type: data.backup_type,
		backup_id: data.backup_id,
	    });
	    win.on('destroy', this.reload, this);
	    win.show();
	},

	onVerify: function() {
	    var view = this.getView();

	    if (!view.datastore) return;

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;

	    let params;

	    if (data.leaf) {
		params = {
		    "backup-type": data["backup-type"],
		    "backup-id": data["backup-id"],
		    "backup-time": (data['backup-time'].getTime()/1000).toFixed(0),
		};
	    } else {
		params = {
		    "backup-type": data.backup_type,
		    "backup-id": data.backup_id,
		};
	    }

	    Proxmox.Utils.API2Request({
		params: params,
		url: `/admin/datastore/${view.datastore}/verify`,
		method: 'POST',
		failure: function(response) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, options) {
		    Ext.create('Proxmox.window.TaskViewer', {
			upid: response.result.data,
		    }).show();
		},
	    });
	},

	onForget: function() {
	    var view = this.getView();

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;
	    if (!data.leaf) return;

	    if (!view.datastore) return;

	    console.log(data);

	    Proxmox.Utils.API2Request({
		params: {
		    "backup-type": data["backup-type"],
		    "backup-id": data["backup-id"],
		    "backup-time": (data['backup-time'].getTime()/1000).toFixed(0),
		},
		url: `/admin/datastore/${view.datastore}/snapshots`,
		method: 'DELETE',
		waitMsgTarget: view,
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		callback: this.reload.bind(this),
	    });
	},

	openBackupFileDownloader: function() {
	    let me = this;
	    let view = me.getView();

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;

	    Ext.create('PBS.window.BackupFileDownloader', {
		baseurl: `/api2/json/admin/datastore/${view.datastore}`,
		params: {
		    'backup-id': data['backup-id'],
		    'backup-type': data['backup-type'],
		    'backup-time': (data['backup-time'].getTime()/1000).toFixed(0),
		},
		files: data.files,
	    }).show();
	},

	openPxarBrowser: function() {
	    let me = this;
	    let view = me.getView();

	    let rec = view.selModel.getSelection()[0];
	    if (!(rec && rec.data)) return;
	    let data = rec.data;

	    let encrypted = false;
	    data.files.forEach(file => {
		if (file.filename === 'catalog.pcat1.didx' && file['crypt-mode'] === 'encrypt') {
		    encrypted = true;
		}
	    });

	    if (encrypted) {
		Ext.Msg.alert(
		    gettext('Cannot open Catalog'),
		    gettext('Only unencrypted Backups can be opened on the server. Please use the client with the decryption key instead.'),
		);
		return;
	    }

	    let id = data['backup-id'];
	    let time = data['backup-time'];
	    let type = data['backup-type'];
	    let timetext = PBS.Utils.render_datetime_utc(data["backup-time"]);

	    Ext.create('PBS.window.FileBrowser', {
		title: `${type}/${id}/${timetext}`,
		datastore: view.datastore,
		'backup-id': id,
		'backup-time': (time.getTime()/1000).toFixed(0),
		'backup-type': type,
	    }).show();
	}
    },

    columns: [
	{
	    xtype: 'treecolumn',
	    header: gettext("Backup Group"),
	    dataIndex: 'text',
	    flex: 1
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Backup Time'),
	    sortable: true,
	    dataIndex: 'backup-time',
	    format: 'Y-m-d H:i:s',
	    width: 150
	},
	{
	    header: gettext("Size"),
	    sortable: true,
	    dataIndex: 'size',
	    renderer: (v, meta, record) => {
		if (v === undefined || v === null) {
		    meta.tdCls = "x-grid-row-loading";
		    return '';
		}
		return Proxmox.Utils.format_size(v);
	    },
	},
	{
	    xtype: 'numbercolumn',
	    format: '0',
	    header: gettext("Count"),
	    sortable: true,
	    dataIndex: 'count',
	},
	{
	    header: gettext("Owner"),
	    sortable: true,
	    dataIndex: 'owner',
	},
	{
	    header: gettext('Encrypted'),
	    dataIndex: 'crypt-mode',
	    renderer: value => PBS.Utils.cryptText[value] || Proxmox.Utils.unknownText,
	},
	{
	    header: gettext("Files"),
	    sortable: false,
	    dataIndex: 'files',
	    renderer: function(files) {
		return files.map((file) => {
		    let icon = '';
		    let size = '';
		    let mode = PBS.Utils.cryptmap.indexOf(file['crypt-mode']);
		    let iconCls = PBS.Utils.cryptIconCls[mode] || '';
		    if (iconCls !== '') {
			icon = `<i class="fa fa-${iconCls}"></i> `;
		    }
		    if (file.size)  {
			size = ` (${Proxmox.Utils.format_size(file.size)})`;
		    }
		    return `${icon}${file.filename}${size}`;
		}).join(', ');
	    },
	    flex: 2
	},
    ],

    tbar: [
	{
	    text: gettext('Reload'),
	    iconCls: 'fa fa-refresh',
	    handler: 'reload',
	},
	'-',
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Verify'),
	    disabled: true,
	    parentXType: 'pbsDataStoreContent',
	    enableFn: (rec) => !!rec.data && rec.data.size !== null,
	    handler: 'onVerify',
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Prune'),
	    disabled: true,
	    parentXType: 'pbsDataStoreContent',
	    enableFn: (rec) => !rec.data.leaf,
	    handler: 'onPrune',
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Forget'),
	    disabled: true,
	    parentXType: 'pbsDataStoreContent',
	    handler: 'onForget',
	    dangerous: true,
	    confirmMsg: function(record) {
		//console.log(record);
		let name = record.data.text;
		return Ext.String.format(gettext('Are you sure you want to remove snapshot {0}'), `'${name}'`);
	    },
	    enableFn: (rec) => !!rec.data.leaf && rec.data.size !== null,
	},
	'-',
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Download Files'),
	    disabled: true,
	    parentXType: 'pbsDataStoreContent',
	    handler: 'openBackupFileDownloader',
	    enableFn: (rec) => !!rec.data.leaf && rec.data.size !== null,
	},
	{
	    xtype: "proxmoxButton",
	    text: gettext('PXAR File Browser'),
	    disabled: true,
	    handler: 'openPxarBrowser',
	    parentXType: 'pbsDataStoreContent',
	    enableFn: function(record) {
		return !!record.data.leaf && record.size !== null && record.data.files.some(el => el.filename.endsWith('pxar.didx'));
	    },
	}
    ],
});
Ext.define('PBS.DataStorePanel', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pbsDataStorePanel',
    mixins: ['Proxmox.Mixin.CBind'],

    cbindData: function(initalConfig) {
	let me = this;
	return {
	    aclPath: `/datastore/${me.datastore}`,
	};
    },

    border: false,
    defaults: {
	border: false,
    },

    items: [
	{
	    xtype: 'pbsDataStoreContent',
	    cbind: {
		datastore: '{datastore}',
	    },
	},
	{
	    xtype: 'pbsDataStoreStatistic',
	    cbind: {
		datastore: '{datastore}',
	    },
	},
	{
	    itemId: 'acl',
	    xtype: 'pbsACLView',
	    aclExact: true,
	    cbind: {
		aclPath: '{aclPath}',
	    },
	},
    ],

    initComponent: function() {
	let me = this;
	me.title = `${gettext("Datastore")}: ${me.datastore}`;
	me.callParent();
    },
});
Ext.define('pve-rrd-node', {
    extend: 'Ext.data.Model',
    fields: [
	{
	    name: 'cpu',
	    // percentage
	    convert: function(value) {
		return value*100;
	    }
	},
	{
	    name: 'iowait',
	    // percentage
	    convert: function(value) {
		return value*100;
	    }
	},
	'netin',
	'netout',
	'memtotal',
	'memused',
	'swaptotal',
	'swapused',
	'total',
	'used',
	'read_ios',
	'read_bytes',
        'write_ios',
	'write_bytes',
        'io_ticks',
	{
	    name: 'io_delay', calculate: function(data) {
		let ios = 0;
		if (data.read_ios !== undefined) { ios += data.read_ios; }
		if (data.write_ios !== undefined) { ios += data.write_ios; }
		if (ios == 0 || data.io_ticks === undefined) {
		    return undefined;
		}
		return (data.io_ticks*1000.0)/ios;
	    }
	},
	'loadavg',
	{ type: 'date', dateFormat: 'timestamp', name: 'time' }
    ]
});
Ext.define('PBS.ServerStatus', {
    extend: 'Ext.panel.Panel',
    alias: 'widget.pbsServerStatus',

    title: gettext('ServerStatus'),

    scrollable: true,

    initComponent: function() {
        var me = this;

	var node_command = function(cmd) {
	    Proxmox.Utils.API2Request({
		params: { command: cmd },
		url: '/nodes/localhost/status',
		method: 'POST',
		waitMsgTarget: me,
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	};

	var restartBtn = Ext.create('Proxmox.button.Button', {
	    text: gettext('Reboot'),
	    dangerous: true,
	    confirmMsg: gettext("Reboot backup server?"),
	    handler: function() {
		node_command('reboot');
	    },
	    iconCls: 'fa fa-undo'
	});

	var shutdownBtn = Ext.create('Proxmox.button.Button', {
	    text: gettext('Shutdown'),
	    dangerous: true,
	    confirmMsg: gettext("Shutdown backup server?"),
	    handler: function() {
		node_command('shutdown');
	    },
	    iconCls: 'fa fa-power-off'
	});

	var consoleBtn = Ext.create('Proxmox.button.Button', {
	    text: gettext('Console'),
	    iconCls: 'fa fa-terminal',
	    handler: function() {
		Proxmox.Utils.openXtermJsViewer('shell', 0, Proxmox.NodeName);
	    }
	});

	me.tbar = [ consoleBtn, restartBtn, shutdownBtn, '->', { xtype: 'proxmoxRRDTypeSelector' } ];

	var rrdstore = Ext.create('Proxmox.data.RRDStore', {
	    rrdurl: "/api2/json/nodes/localhost/rrd",
	    model: 'pve-rrd-node'
	});

	me.items = {
	    xtype: 'container',
	    itemId: 'itemcontainer',
	    layout: 'column',
	    minWidth: 700,
	    defaults: {
		minHeight: 320,
		padding: 5,
		columnWidth: 1
	    },
	    items: [
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('CPU usage'),
		    fields: ['cpu','iowait'],
		    fieldTitles: [gettext('CPU usage'), gettext('IO wait')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Server load'),
		    fields: ['loadavg'],
		    fieldTitles: [gettext('Load average')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Memory usage'),
		    fields: ['memtotal','memused'],
		    fieldTitles: [gettext('Total'), gettext('RAM usage')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Swap usage'),
		    fields: ['swaptotal','swapused'],
		    fieldTitles: [gettext('Total'), gettext('Swap usage')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Network traffic'),
		    fields: ['netin','netout'],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Root Disk usage'),
		    fields: ['total','used'],
		    fieldTitles: [gettext('Total'), gettext('Disk usage')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Root Disk Transfer Rate (bytes/second)'),
		    fields: ['read_bytes','write_bytes'],
		    fieldTitles: [gettext('Read'), gettext('Write')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Root Disk Input/Output Operations per Second (IOPS)'),
		    fields: ['read_ios','write_ios'],
		    fieldTitles: [gettext('Read'), gettext('Write')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Root Disk IO Delay (ms)'),
		    fields: ['io_delay'],
		    fieldTitles: [gettext('IO Delay')],
		    store: rrdstore
		},
	    ]
	};

	me.listeners = {
	    activate: function() {
		rrdstore.startUpdate();
	    },
	    destroy: function() {
		rrdstore.stopUpdate();
	    },
	};

	me.callParent();
    }

});
/*global Proxmox*/
Ext.define('PBS.ServerAdministration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pbsServerAdministration',

    title: gettext('Server Administration'),

    border: true,
    defaults: { border: false },

    controller: {
	xclass: 'Ext.app.ViewController',

        init: function(view) {
	    var upgradeBtn = view.lookupReference('upgradeBtn');
	    upgradeBtn.setDisabled(!(Proxmox.UserName && Proxmox.UserName === 'root@pam'));
	}
    },

    items: [
	{
	    xtype: 'pbsServerStatus',
	    itemId: 'status'
	},
	{
	    xtype: 'proxmoxNodeServiceView',
            title: gettext('Services'),
	    itemId: 'services',
	    startOnlyServices: {
		syslog: true,
		'proxmox-backup': true,
		'proxmox-backup-proxy': true,
	    },
	    nodename: 'localhost'
	},
	{
	    xtype: 'proxmoxNodeAPT',
            title: gettext('Updates'),
	    upgradeBtn: {
		xtype: 'button',
		reference: 'upgradeBtn',
		disabled: true,
		text: gettext('Upgrade'),
		handler: function() {
		    Proxmox.Utils.openXtermJsViewer('upgrade', 0, 'localhost');
		}
	    },
	    itemId: 'updates',
	    nodename: 'localhost'
	},
	{
	    xtype: 'proxmoxJournalView',
	    itemId: 'logs',
	    title: gettext('Syslog'),
	    url: "/api2/extjs/nodes/localhost/journal",
	},
	{
	    xtype: 'proxmoxNodeTasks',
	    itemId: 'tasks',
	    title: gettext('Tasks'),
	    height: 'auto',
	    nodename: 'localhost'
	}
    ]
});


Ext.define('PBS.Dashboard', {
    extend: 'Ext.panel.Panel',
    xtype: 'pbsDashboard',

    controller: {
	xclass: 'Ext.app.ViewController',

	openDashboardOptions: function() {
	    var me = this;
	    var viewModel = me.getViewModel();
	    Ext.create('Ext.window.Window', {
		modal: true,
		width: 300,
		title: gettext('Dashboard Options'),
		layout: {
		    type: 'auto'
		},
		items: [{
		    xtype: 'form',
		    bodyPadding: '10 10 10 10',
		    defaultButton: 'savebutton',
		    items: [{
			xtype: 'proxmoxintegerfield',
			itemId: 'hours',
			labelWidth: 100,
			anchor: '100%',
			allowBlank: false,
			minValue: 1,
			maxValue: 24,
			value: viewModel.get('hours'),
			fieldLabel: gettext('Hours to show')
		    }],
		    buttons: [{
			text: gettext('Save'),
			reference: 'loginButton',
			formBind: true,
			handler: function() {
			    var win = this.up('window');
			    var hours = win.down('#hours').getValue();
			    me.setHours(hours, true);
			    win.close();
			}
		    }]
		}]
	    }).show();
	},

	setHours: function(hours, setState) {
	    var me = this;
	    var viewModel = me.getViewModel();
	    viewModel.set('hours', hours);
	    viewModel.notify();

	    if (setState) {
		var sp = Ext.state.Manager.getProvider();
		sp.set('dashboard-hours', hours);
	    }
	},


	updateSubscription: function(store, records, success) {
	    if (!success) { return; }
	    let me = this;
	    let subStatus = records[0].data.status === 'Active' ? 2 : 0; // 2 = all good, 1 = different leves, 0 = none
	    me.lookup('subscription').setSubStatus(subStatus);
	},

	updateUsageStats: function(store, records, success) {
	    if (!success) {
		return;
	    }
	    if (records === undefined || records.length < 1) {
		return;
	    }
	    let me = this;
	    let viewmodel = me.getViewModel();

	    let res = records[0].data;
	    viewmodel.set('fingerprint', res.info.fingerprint || Proxmox.Utils.unknownText);

	    let cpu = res.cpu,
	        mem = res.memory,
	        root = res.root;

	    var cpuPanel = me.lookup('cpu');
	    cpuPanel.updateValue(cpu);

	    var memPanel = me.lookup('mem');
	    memPanel.updateValue(mem.used / mem.total);

	    var hdPanel = me.lookup('root');
	    hdPanel.updateValue(root.used / root.total);
	},

	showFingerPrint: function() {
	    let me = this;
	    let vm = me.getViewModel();
	    let fingerprint = vm.get('fingerprint');
	    Ext.create('Ext.window.Window', {
		modal: true,
		width: 600,
		title: gettext('Fingerprint'),
		layout: 'form',
		bodyPadding: '10 0',
		items: [
		    {
			xtype: 'textfield',
			inputId: 'fingerprintField',
			value: fingerprint,
			editable: false,
		    },
		],
		buttons: [
		    {
			xtype: 'button',
			iconCls: 'fa fa-clipboard',
			handler: function(b) {
			    var el = document.getElementById('fingerprintField');
			    el.select();
			    document.execCommand("copy");
			},
			text: gettext('Copy')
		    },
		    {
			text: gettext('Ok'),
			handler: function() {
			    this.up('window').close();
			},
		    },
		],
	    }).show();
	},

	updateTasks: function(store, records, success) {
	    if (!success) return;
	    let me = this;

	    records.sort((a, b) => a.data.duration - b.data.duration);
	    let top10 = records.slice(-10);
	    me.lookup('longesttasks').updateTasks(top10);

	    let data = {
		backup: {  error: 0, warning: 0, ok: 0, },
		prune: { error: 0, warning: 0, ok: 0, },
		garbage_collection: { error: 0, warning: 0, ok: 0, },
		sync: {  error: 0, warning: 0, ok: 0, },
	    };

	    records.forEach(record => {
		let type = record.data.worker_type;
		if (type === 'syncjob') {
		    type = 'sync';
		}

		if (data[type] && record.data.status) {
		    let parsed = Proxmox.Utils.parse_task_status(record.data.status);
		    data[type][parsed]++;
		}
	    });

	    me.lookup('tasksummary').updateTasks(data);
	},

	init: function(view) {
	    var me = this;
	    var sp = Ext.state.Manager.getProvider();
	    var hours = sp.get('dashboard-hours') || 12;
	    me.setHours(hours, false);
	}
    },

    viewModel: {
	data: {
	    timespan: 300, // in seconds
	    hours: 12, // in hours
	    error_shown: false,
	    fingerprint: "",
	    'bytes_in': 0,
	    'bytes_out': 0,
	    'avg_ptime': 0.0
	},

	formulas: {
	    disableFPButton: (get) => get('fingerprint') === "",
	},

	stores: {
	    usage: {
		storeid: 'dash-usage',
		type: 'update',
		interval: 3000,
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/nodes/localhost/status'
		},
		listeners: {
		    load: 'updateUsageStats'
		}
	    },
	    subscription: {
		storeid: 'dash-subscription',
		type: 'update',
		interval: 10000,
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/nodes/localhost/subscription'
		},
		listeners: {
		    load: 'updateSubscription'
		}
	    },
	    tasks: {
		storeid: 'dash-tasks',
		type: 'update',
		interval: 15000,
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		model: 'proxmox-tasks',
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/status/tasks'
		},
		listeners: {
		    load: 'updateTasks'
		}
	    },
	}
    },

    title: gettext('Dashboard') + ' - WIP',

    layout: {
	type: 'column'
    },

    bodyPadding: '20 0 0 20',

    defaults: {
	columnWidth: 0.49,
	xtype: 'panel',
	margin: '0 20 20 0'
    },

    scrollable: true,

    items: [
	{
	    height: 250,
	    iconCls: 'fa fa-tasks',
	    title: gettext('Server Resources'),
	    bodyPadding: '0 20 0 20',
	    tools: [
		{
		    xtype: 'button',
		    text: gettext('Show Fingerprint'),
		    handler: 'showFingerPrint',
		    bind: {
			disabled: '{disableFPButton}',
		    },
		},
	    ],
	    layout: {
		type: 'hbox',
		align: 'center'
	    },
	    defaults: {
		xtype: 'proxmoxGauge',
		spriteFontSize: '20px',
		flex: 1
	    },
	    items: [
		{
		    title: gettext('CPU'),
		    reference: 'cpu'
		},
		{
		    title: gettext('Memory'),
		    reference: 'mem'
		},
		{
		    title: gettext('Root Disk'),
		    reference: 'root'
		}
	    ]
	},
	{
	    xtype: 'pbsDatastoresStatistics',
	    height: 250,
	},
	{
	    xtype: 'pbsLongestTasks',
	    reference: 'longesttasks',
	    height: 250,
	},
	{
	    xtype: 'pbsRunningTasks',
	    height: 250,
	},
	{
	    xtype: 'pbsTaskSummary',
	    reference: 'tasksummary',
	},
	{
	    iconCls: 'fa fa-ticket',
	    title: 'Subscription',
	    height: 166,
	    reference: 'subscription',
	    xtype: 'pbsSubscriptionInfo',
	},
    ]
});

Ext.define('PBS.dashboard.SubscriptionInfo', {
    extend: 'Ext.panel.Panel',
    xtype: 'pbsSubscriptionInfo',

    style: {
	cursor: 'pointer'
    },

    layout: {
	type: 'hbox',
	align: 'middle',
    },

    items: [
	{
	    xtype: 'box',
	    itemId: 'icon',
	    data: {
		icon: 'question-circle',
	    },
	    width: 100,
	    tpl: '<center><i class="fa fa-3x fa-{icon}"></i></center>',
	},
	{
	    flex: 1,
	    xtype: 'box',
	    data: {
		message: gettext('Unknown'),
	    },
	    itemId: 'message',
	    tpl: '<center>{message}</center>',
	},
    ],

    setSubStatus: function(status) {
	var me = this;
	let icon = '';
	let message = '';

	switch (status) {
	    case 2:
		icon = 'check good';
		message = gettext('Your subscription status is valid.');
		break;
	    case 1:
		icon = 'exclamation-triangle warning';
		message = gettext('Warning: Your subscription levels are not the same.');
		break;
	    case 0:
		icon = 'times-circle critical';
		message = gettext('This node does not have a subscription.');
		break;
	    default:
		throw 'invalid subscription status';
	}
	me.getComponent('icon').update({ icon });
	me.getComponent('message').update({ message });
    },

    listeners: {
	click: {
	    element: 'body',
	    fn: function() {
		var mainview = this.component.up('mainview');
		mainview.getController().redirectTo('pbsSubscription');
	    }
	}
    }
});
Ext.define('PBS.store.NavigationStore', {
    extend: 'Ext.data.TreeStore',

    storeId: 'NavigationStore',

    root: {
	expanded: true,
	children: [
	    {
		text: gettext('Dashboard'),
		iconCls: 'fa fa-tachometer',
		path: 'pbsDashboard',
		leaf: true
	    },
	    {
		text: gettext('Configuration'),
		iconCls: 'fa fa-gears',
		path: 'pbsSystemConfiguration',
		expanded: true,
		children: [
		    {
			text: gettext('User Management'),
			iconCls: 'fa fa-user',
			path: 'pbsUserView',
			leaf: true
		    },
		    {
			text: gettext('Permissions'),
			iconCls: 'fa fa-unlock',
			path: 'pbsACLView',
			leaf: true
		    },
		    {
			text: gettext('Remotes'),
			iconCls: 'fa fa-server',
			path: 'pbsRemoteView',
			leaf: true,
		    },
		    {
			text: gettext('Sync Jobs'),
			iconCls: 'fa fa-refresh',
			path: 'pbsSyncJobView',
			leaf: true,
		    },
		    {
			text: gettext('Subscription'),
			iconCls: 'fa fa-support',
			path: 'pbsSubscription',
			leaf: true
		    }
		]
	    },
	    {
		text: gettext('Administration'),
		iconCls: 'fa fa-wrench',
		path: 'pbsServerAdministration',
		expanded: true,
		leaf: false,
		children: [
		    {
			text: gettext('Disks'),
			iconCls: 'fa fa-hdd-o',
			path: 'pmxDiskList',
			leaf: false,
			children: [
			    {
				text: Proxmox.Utils.directoryText,
				iconCls: 'fa fa-folder',
				path: 'pbsDirectoryList',
				leaf: true,
			    },
			    {
				text: "ZFS",
				iconCls: 'fa fa-th-large',
				path: 'pbsZFSList',
				leaf: true,
			    },
			]
		    }
		]
	    },
	    {
		text: gettext('Datastore'),
		iconCls: 'fa fa-archive',
		path: 'pbsDataStoreConfig',
		expanded: true,
		leaf: false
	    },
	]
    }
});

Ext.define('PBS.view.main.NavigationTree', {
    extend: 'Ext.list.Tree',
    xtype: 'navigationtree',

    controller: {
	xclass: 'Ext.app.ViewController',

	init: function(view) {

	    view.rstore = Ext.create('Proxmox.data.UpdateStore', {
		autoStart: true,
		interval: 15 * 1000,
		storeid: 'pbs-datastore-list',
		model: 'pbs-datastore-list'
	    });

	    view.rstore.on('load', this.onLoad, this);
	    view.on('destroy', view.rstore.stopUpdate);
	},

	onLoad: function(store, records, success) {
	    if (!success) return;
	    var view = this.getView();

	    let root = view.getStore().getRoot();

	    // FIXME: newly added always get appended to the end..
	    records.sort((a, b) => {
		if (a.id > b.id) return  1;
		if (a.id < b.id) return -1;
		return 0;
	    });

	    var list = root.findChild('path', 'pbsDataStoreConfig', false);
	    var length = records.length;
	    var lookup_hash = {};
	    for (var i = 0; i < length; i++) {
		var name = records[i].id;
		lookup_hash[name] = true;
		if (!list.findChild('text', name, false)) {
		    list.appendChild({
			text: name,
			path: `DataStore-${name}`,
			iconCls: 'fa fa-database',
			leaf: true
		    });
		}
	    }

	    var erase_list = [];
	    list.eachChild(function(node) {
		var name = node.data.text;
		if (!lookup_hash[name]) {
		    erase_list.push(node);
		}
	    });

	    Ext.Array.forEach(erase_list, function(node) { node.erase(); });

	}
    },

    select: function(path) {
	var me = this;
	var item = me.getStore().findRecord('path', path, 0, false, true, true);
	me.setSelection(item);
    },

    animation: false,
    expanderOnly: true,
    expanderFirst: false,
    store: 'NavigationStore',
    ui: 'nav'
});
/*global Proxmox*/
Ext.define('PBS.Application', {
    extend: 'Ext.app.Application',

    name: 'PBS',
    appProperty: 'app',

    stores: [
	'NavigationStore'
    ],

    layout: 'fit',

    realignWindows: function() {
	var modalwindows = Ext.ComponentQuery.query('window[modal]');
	Ext.Array.forEach(modalwindows, function(item) {
	    item.center();
	});
    },

    logout: function() {
	var me = this;
	Proxmox.Utils.authClear();
	me.changeView('loginview', true);
    },

    changeView: function(view, skipCheck) {
	var me = this;
	PBS.view = view;
	me.view = view;

	if (me.currentView != undefined) {
	    me.currentView.destroy();
	}

	me.currentView = Ext.create({
	    xtype: view,
	});
	if (skipCheck !== true) {
	    // fixme:
	    // Proxmox.Utils.checked_command(function() {}); // display subscription status
	}
    },

    view: 'loginview',

    launch: function() {
	var me = this;
	Ext.on('resize', me.realignWindows);

	var provider = new Ext.state.LocalStorageProvider({ prefix: 'ext-pbs-' });
	Ext.state.Manager.setProvider(provider);

	// show login window if not loggedin
	var loggedin = Proxmox.Utils.authOK();
	if (!loggedin) {
	    me.changeView('loginview', true);
	} else {
	    me.changeView('mainview', true);
	}
    }
});

Ext.application('PBS.Application');
Ext.define('PBS.MainView', {
    extend: 'Ext.container.Container',
    xtype: 'mainview',

    title: 'Proxmox Backup Server',

    controller: {
	xclass: 'Ext.app.ViewController',
	routes: {
	    ':path:subpath': {
		action: 'changePath',
		before: 'beforeChangePath',
                conditions : {
		    ':path'    : '(?:([%a-zA-Z0-9\\-\\_\\s,\.]+))',
		    ':subpath' : '(?:(?::)([%a-zA-Z0-9\\-\\_\\s,]+))?'
		}
	    }
	},

	beforeChangePath: function(path, subpath, action) {
	    var me = this;

	    let xtype = path;
	    let datastore;
	    let isDataStore = PBS.Utils.isDataStorePath(path);
	    if (isDataStore) {
		xtype = 'pbsDataStorePanel';
		datastore = PBS.Utils.getDataStoreFromPath(path);
	    }

	    if (!Ext.ClassManager.getByAlias(`widget.${xtype}`)) {
		console.warn(`xtype ${xtype} not found`);
		action.stop();
		return;
	    }

	    var lastpanel = me.lookupReference('contentpanel').getLayout().getActiveItem();
	    if (lastpanel && lastpanel.xtype === xtype) {
		if (isDataStore) {
		    if (datastore === lastpanel.datastore) {
			action.stop();
			return;
		    }
		} else {
		    // we have the right component already,
		    // we just need to select the correct tab
		    // default to the first
		    subpath = subpath || 0;
		    if (lastpanel.getActiveTab) {
			// we assume lastpanel is a tabpanel
			if (lastpanel.getActiveTab().getItemId() !== subpath) {
			    // set the active tab
			    lastpanel.setActiveTab(subpath);
			}
			// else we are already there
		    }
		    action.stop();
		    return;
		}
	    }

	    action.resume();
	},

	changePath: function(path, subpath) {
	    var me = this;
	    var contentpanel = me.lookupReference('contentpanel');
	    var lastpanel = contentpanel.getLayout().getActiveItem();

	    var obj;
	    if (PBS.Utils.isDataStorePath(path)) {
		let datastore = PBS.Utils.getDataStoreFromPath(path);
		obj = contentpanel.add({
		    xtype: 'pbsDataStorePanel',
		    nodename: 'localhost',
		    datastore,
		});
	    } else {
		obj = contentpanel.add({
		    xtype: path,
		    nodename: 'localhost',
		    border: false
		});
	    }

	    var treelist = me.lookupReference('navtree');

	    treelist.suspendEvents();
	    if (subpath === undefined) {
		treelist.select(path);
	    } else {
		treelist.select(path + ':' + subpath);
	    }
	    treelist.resumeEvents();

	    if (Ext.isFunction(obj.setActiveTab)) {
		obj.setActiveTab(subpath || 0);
		obj.addListener('tabchange', function(tabpanel, newc, oldc) {
		    var newpath = path;

		    // only add the subpath part for the
		    // non-default tabs
		    if (tabpanel.items.findIndex('id', newc.id) !== 0) {
			newpath += ":" + newc.getItemId();
		    }

		    me.redirectTo(newpath);
		});
	    }

	    contentpanel.setActiveItem(obj);

	    if (lastpanel) {
		contentpanel.remove(lastpanel, { destroy: true });
	    }

	},

	logout: function() {
	    PBS.app.logout();
	},

	navigate: function(treelist, item) {
	    this.redirectTo(item.get('path'));
	},

	control: {
	    '[reference=logoutButton]': {
		click: 'logout'
	    }
	},

	init: function(view) {
	    var me = this;

	    PBS.data.RunningTasksStore.startUpdate();
	    me.lookupReference('usernameinfo').setText(Proxmox.UserName);

	    // show login on requestexception
	    // fixme: what about other errors
	    Ext.Ajax.on('requestexception', function(conn, response, options) {
		if (response.status == 401) { // auth failure
		    me.logout();
		}
	    });

	    // get ticket periodically
	    Ext.TaskManager.start({
		run: function() {
		    var ticket = Proxmox.Utils.authOK();
		    if (!ticket || !Proxmox.UserName) {
			return;
		    }

		    Ext.Ajax.request({
			params: {
			    username: Proxmox.UserName,
			    password: ticket
			},
			url: '/api2/json/access/ticket',
			method: 'POST',
			failure: function() {
			    me.logout();
			},
			success: function(response, opts) {
			    var obj = Ext.decode(response.responseText);
			    PBS.Utils.updateLoginData(obj.data);
			}
		    });
		},
		interval: 15*60*1000
	    });


	    // select treeitem and load page from url fragment, if set
	    let token = Ext.util.History.getToken() || 'pbsDashboard';
	    this.redirectTo(token, true);
	}
    },

    plugins: 'viewport',

    layout: { type: 'border' },

    items: [
	{
	    region: 'north',
	    xtype: 'container',
	    layout: {
		type: 'hbox',
		align: 'middle'
	    },
	    margin: '2 0 2 5',
	    height: 38,
	    items: [
		{
		    xtype: 'proxmoxlogo',
		    prefix: '',
		},
		{
		    padding: '0 0 0 5',
		    xtype: 'versioninfo',
		},
		{
		    padding: 5,
		    html: '<a href="https://bugzilla.proxmox.com" target="_blank">BETA</a>',
		    baseCls: 'x-plain',
		},
		{
		    flex: 1,
		    baseCls: 'x-plain',
		},
		{
		    xtype: 'button',
		    baseCls: 'x-btn',
		    cls: 'x-btn-default-toolbar-small proxmox-inline-button',
		    iconCls: 'fa fa-book x-btn-icon-el-default-toolbar-small ',
		    text: gettext('Documentation'),
		    href: '/docs/index.html',
		    margin: '0 5 0 0',
		},
		{
		    xtype: 'pbsTaskButton',
		    margin: '0 5 0 0',
		},
		{
		    xtype: 'button',
		    reference: 'usernameinfo',
		    style: {
			// proxmox dark grey p light grey as border
			backgroundColor: '#464d4d',
			borderColor: '#ABBABA'
		    },
		    margin: '0 5 0 0',
		    iconCls: 'fa fa-user',
		    menu: [
			{
			    reference: 'logoutButton',
			    iconCls: 'fa fa-sign-out',
			    text: gettext('Logout'),
			},
		    ],
		},
	    ]
	},
	{
	    xtype: 'panel',
	    scrollable: 'y',
	    border: false,
	    region: 'west',
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    items: [{
		xtype: 'navigationtree',
		minWidth: 180,
		reference: 'navtree',
		// we have to define it here until extjs 6.2
		// because of a bug where a viewcontroller does not detect
		// the selectionchange event of a treelist
		listeners: {
		    selectionchange: 'navigate'
		}
	    }, {
		xtype: 'box',
		cls: 'x-treelist-nav',
		flex: 1
	    }]
	},
	{
	    xtype: 'panel',
	    layout: { type: 'card' },
	    region: 'center',
	    border: false,
	    reference: 'contentpanel'
	}
    ]
});
